#include <gtk/gtk.h>


enum {
  TARGET_STRING,
  TARGET_ROOTWIN
};

static GtkTargetEntry target_table[] = {
  { "STRING",     0, TARGET_STRING },
  { "text/plain", 0, TARGET_STRING },
  { "application/x-rootwin-drop", 0, TARGET_ROOTWIN }
};

static guint n_targets = sizeof(target_table) / sizeof(target_table[0]);

void  
label_drag_data_received  (GtkWidget          *widget,
			    GdkDragContext     *context,
			    gint                x,
			    gint                y,
			    GtkSelectionData   *data,
			    guint               info,
			    guint               time)
{
  if ((data->length >= 0) && (data->format == 8))
    {
      g_print ("Received \"%s\" in label\n", (gchar *)data->data);
      gtk_drag_finish (context, TRUE, FALSE, time);
      return;
    }
  
  gtk_drag_finish (context, FALSE, FALSE, time);
}

void  
label_drag_motion  (GtkWidget          *widget,
		    GdkDragContext     *context,
		    gint                x,
		    gint                y,
		    GtkSelectionData   *data,
		    guint               info,
		    guint               time)
{
	g_print("Motion\n");
}

void  
source_drag_data_get  (GtkWidget          *widget,
		       GdkDragContext     *context,
		       GtkSelectionData   *selection_data,
		       guint               info,
		       guint               time,
		       gpointer            data)
{
  if (info == TARGET_ROOTWIN)
    g_print ("I was dropped on the rootwin\n");
  else {
    gtk_selection_data_set (selection_data,
			    selection_data->target,
			    8, "I'm Data!", 9);
  }
}

void  
source_drag_data_delete  (GtkWidget          *widget,
			  GdkDragContext     *context,
			  gpointer            data)
{
  g_print ("Delete the data!\n");
}

static void
source_drag_begin (GtkWidget      *widget,
		   GdkDragContext *context,
		   GtkWidget *mainwin) {
	GdkCursor *hand;

	hand=gdk_cursor_new(GDK_HAND2);
	gdk_window_set_cursor (mainwin->window, hand);
/*	gtk_drag_set_icon_widget (context, widget, 0, 0);
	printf("Start\n");*/
}

main(int argc, char *argv[]) {
	GtkWidget *win;
	GtkWidget *hbox, *label, *list;
	char *txt[1], a[4]="abc", b[4]="def", c[4]="ghi";

	gtk_init(&argc, &argv);
	
	win=gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect (GTK_OBJECT (win), "destroy",
			    GTK_SIGNAL_FUNC(gtk_main_quit),
			    NULL);
	gtk_window_set_title (GTK_WINDOW (win), "Drag -N- Drop");
	gtk_widget_set_usize(win, 200, 100);
	gtk_container_border_width (GTK_CONTAINER (win), 0);

	hbox=gtk_hbox_new(FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (hbox), 10);
	gtk_container_add(GTK_CONTAINER(win), hbox);
	gtk_widget_show (hbox);

	label=gtk_label_new("?");
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	
	list=gtk_clist_new(1);
	gtk_box_pack_start (GTK_BOX (hbox), list, TRUE, TRUE, 0);
	gtk_widget_show (list);

	txt[0]=a;
	gtk_clist_append(GTK_CLIST(list), txt);
	txt[0]=b;
	gtk_clist_append(GTK_CLIST(list), txt);
	txt[0]=c;
	gtk_clist_append(GTK_CLIST(list), txt);
	
	/* Drag & Drop */
	gtk_drag_dest_set (label,
		     GTK_DEST_DEFAULT_ALL,
		     target_table, n_targets - 1, /* no rootwin */
		     GDK_ACTION_COPY | GDK_ACTION_MOVE);
	gtk_signal_connect( GTK_OBJECT(label), "drag_data_received",
		      GTK_SIGNAL_FUNC(label_drag_data_received), NULL);
	gtk_signal_connect( GTK_OBJECT(label), "drag_motion",
		      GTK_SIGNAL_FUNC(label_drag_motion), NULL);

	gtk_drag_source_set (list, GDK_BUTTON1_MASK,
		       target_table, n_targets, 
		       GDK_ACTION_COPY | GDK_ACTION_MOVE);
	gtk_signal_connect (GTK_OBJECT (list), "drag_begin",
			    GTK_SIGNAL_FUNC (source_drag_begin), win);
	gtk_signal_connect (GTK_OBJECT (list), "drag_data_get",
			    GTK_SIGNAL_FUNC (source_drag_data_get), NULL);
	gtk_signal_connect (GTK_OBJECT (list), "drag_data_delete",
			    GTK_SIGNAL_FUNC (source_drag_data_delete), NULL);


	gtk_widget_show(win);
	gtk_main();
}
