/* Copyright (C) Emmanuel Varagnat 1998

   Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
   le modifier conformment aux dispositions de la Licence Publique Gnrale
   GNU, telle que publie par la Free Software Foundation ; version 2 de la
   licence, ou encore ( votre choix) toute version ultrieure.

   Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE
   GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou
   D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence
   Publique Gnrale GNU .

   Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en
   mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
   Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.

  ==========================================================================

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ==========================================================================

   Ecrit par Emmanuel Varagnat (emmanuel.varagnat@altavista.net)  */

#include <gtk/gtk.h>
#include <gtk/gtkclist.h>

#include "arbo.h"
#include "config.h"
#include "explore.h"
#include "files.h"
#include "mv_cmd.h"
#include "popups.h"
#include "rm_cmd.h"

#include <unistd.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

typedef struct rep {
			gchar *name;
			int mode;
			struct rep *suiv;
	} rep_entry;

/* Drag & Drop - DEBUT */

enum {
  TARGET_STRING,
  TARGET_ROOTWIN
};

static GtkTargetEntry target_table[] = {
  { "STRING",     0, TARGET_STRING },
  { "text/plain", 0, TARGET_STRING },
  { "application/x-rootwin-drop", 0, TARGET_ROOTWIN }
};

static guint n_targets = sizeof(target_table) / sizeof(target_table[0]);
GtkCTreeNode *save_row;

/* Drag & Drop - FIN */

extern char *HOMEpath;

static void arbo_drag_motion(GtkWidget*, GdkDragContext*, gint, gint, GtkSelectionData*, guint, guint);
static void arbo_drag_data_received(GtkWidget*, GdkDragContext*, gint, gint, GtkSelectionData*, guint, guint);
static void arbo_button_press();
static void arbo_tree_collapse();
static void arbo_selected(GtkCTree*, GtkCTreeNode*);
static void NewDirItem(GtkCTreeNode*, char*, char*, int, int);
static void CreateBranch(GtkCTreeNode*, char*);
static void arbo_change_dir_recursive(char*, GtkCTreeNode*);
static void arbo_refresh_dir_recursive(char*, GtkCTreeNode*);
static int  subdir(char*);
static int IsPartOfPath(char*, char*);

GtkWidget *arbo;
char select_path[255]="";

GtkCTreeNode *InitArbo(GtkWidget *container) {
	GtkCTreeNode *parent, *poub;
	char *root_txt[1] = {"/"};
	char *poub_txt[1] = {"corbeille"};

	arbo=gtk_ctree_new (1, 0);
	gtk_signal_connect (GTK_OBJECT (arbo), "button_press_event",
                               GTK_SIGNAL_FUNC (arbo_button_press), NULL);
 	gtk_ctree_set_line_style (GTK_CTREE(arbo), GTK_CTREE_LINES_DOTTED);
	gtk_ctree_set_indent (GTK_CTREE(arbo), 17);
	gtk_clist_set_row_height(GTK_CLIST(arbo), 16);
	gtk_container_add (GTK_CONTAINER(container), arbo);
	gtk_widget_show(arbo);

	/* Drag & Drop */
	gtk_drag_dest_set (arbo,
		     GTK_DEST_DEFAULT_ALL,
		     target_table, n_targets - 1, /* no rootwin */
		     GDK_ACTION_COPY | GDK_ACTION_MOVE);
	gtk_signal_connect( GTK_OBJECT(arbo), "drag_data_received",
		      GTK_SIGNAL_FUNC(arbo_drag_data_received), NULL);
	gtk_signal_connect( GTK_OBJECT(arbo), "drag_motion",
		      GTK_SIGNAL_FUNC(arbo_drag_motion), NULL);

	parent = gtk_ctree_insert_node (GTK_CTREE(arbo), NULL, NULL, root_txt, 5, pix_dir,
						msk_dir, pix_dir, msk_dir, FALSE, TRUE);
	gtk_ctree_node_set_row_data (GTK_CTREE(arbo), parent, "/");

	poub=gtk_ctree_insert_node (GTK_CTREE(arbo), NULL, NULL, poub_txt, 5, pix_poub,
					msk_poub, pix_poub, msk_poub, FALSE, TRUE);
	gtk_ctree_node_set_row_data (GTK_CTREE(arbo), poub, "");

	CreateBranch(parent, "/");

	return parent;
}

/********************************************************/
/********************************************************/

static void arbo_drag_motion  (GtkWidget          *widget,
			       GdkDragContext     *context,
			       gint                x,
			       gint                y,
			       GtkSelectionData   *data,
			       guint               info,
			       guint               time)
{
	int row, column;
/*	g_print("Motion (%d, %d)\n", x, y);*/
	gtk_clist_get_selection_info (GTK_CLIST(arbo), x, y, &row, &column);
	if(GTK_CLIST(arbo)->focus_row!=row) {
		GTK_CLIST(arbo)->focus_row=row;
		gtk_signal_emit_by_name (GTK_OBJECT (arbo), "toggle_focus_row");
	}
}

static void arbo_drag_data_received  (GtkWidget          *widget,
				      GdkDragContext     *context,
				      gint                x,
				      gint                y,
				      GtkSelectionData   *data,
				      guint               info,
				      guint               time)
{
  gtk_ctree_select(GTK_CTREE(arbo), save_row);
/*  GTK_CLIST(arbo)->focus_row=save_row;
  gtk_signal_emit_by_name (GTK_OBJECT (arbo), "toggle_focus_row");*/

  if ((data->length >= 0) && (data->format == 8))
    {
      g_print ("Received \"%s\" in label\n", (gchar *)data->data);
      gtk_drag_finish (context, TRUE, FALSE, time);
      return;
    }
  
  gtk_drag_finish (context, FALSE, FALSE, time);
}


static void arbo_button_press(GtkWidget *widget, GdkEventButton *event) {
	GtkCList *clist;
	GtkCTree *ctree;
	GtkCTreeNode *work;
	gint x, y;
	gint row, column;
	gchar *path;

	x = event->x;
	y = event->y;

	clist = GTK_CLIST (widget);
	ctree = GTK_CTREE (widget);
	if (!gtk_clist_get_selection_info (clist, x, y, &row, &column))
		return;

	work = GTK_CTREE_NODE (g_list_nth (clist->row_list, row));

	gtk_signal_emit_stop_by_name(GTK_OBJECT(ctree), "button_press_event");
	
	path=(char*)gtk_ctree_node_get_row_data (ctree, work);
	if(path[0]=='\0')
	      return;

	switch (event->type) {
	   case GDK_2BUTTON_PRESS:
		    if (GTK_CTREE_ROW(work)->expanded)
			    arbo_tree_collapse (ctree, work);
		    else if(GTK_CTREE_ROW (work)->children)
			    arbo_tree_expand (ctree, work);
	   	    break;

	   case GDK_BUTTON_PRESS:
		    switch(event->button) {
		     	case 1:
			     if(gtk_ctree_is_hot_spot (GTK_CTREE(widget), event->x, event->y) ) {
				if (GTK_CTREE_ROW (work)->expanded)
					arbo_tree_collapse (ctree, work);
				else
					arbo_tree_expand (ctree, work);
			     }
			     else
				arbo_selected(ctree, work);
			     break;
/*			case 2:
			     write(1, "%", 1);
			     gtk_tooltips_set_tip (tips, arbo, path, NULL);
			     gtk_tooltips_force_window (tips);
			     break;*/
			case 3:			
			     if(g_list_find(GTK_CLIST(arbo)->selection, (gpointer)work)!=NULL)
			           gtk_menu_popup (GTK_MENU(pop_dir), NULL, NULL,
				   	NULL, NULL, event->button, event->time);
			     break;
			default:
			     break; 
		    }
		    break;
	   default:
	   	    break;
	}
}

void arbo_tree_expand(GtkWidget *widget, gpointer p) {
	GtkCTreeNode *nd=GTK_CTREE_NODE(p);

	gtk_clist_freeze(GTK_CLIST(widget));
	if(GTK_CTREE_ROW(nd)->children)
	    gtk_ctree_remove_node(GTK_CTREE(widget), GTK_CTREE_ROW(nd)->children);
	CreateBranch(nd, (char *)gtk_ctree_node_get_row_data (GTK_CTREE(widget), nd));
	gtk_clist_thaw(GTK_CLIST(widget));
	gtk_ctree_expand (GTK_CTREE(widget), nd);
}

static void arbo_tree_collapse(GtkCTree *ctree, GtkCTreeNode *nd) {
	char *pt[1]={"."};
	
	gtk_ctree_collapse (ctree, nd);
	gtk_clist_freeze(GTK_CLIST(ctree));
	while(GTK_CTREE_ROW(nd)->children) {
		gtk_ctree_remove_node(ctree, GTK_CTREE_ROW(nd)->children);
	}

	if(access((char*)gtk_ctree_node_get_row_data (ctree, nd), R_OK|X_OK)==0)
		gtk_ctree_insert_node (ctree, nd, NULL, pt, 5,
			NULL, NULL, NULL, NULL,
			TRUE, FALSE);
	if(GTK_CLIST (ctree)->selection==NULL)
		gtk_entry_set_text(GTK_ENTRY(chdir_edit), "");

	gtk_clist_thaw(GTK_CLIST(ctree));
}

static void arbo_destroy_path(char *p) {
	free(p);
}

static void arbo_selected(GtkCTree *ctree, GtkCTreeNode *nd) {
	char *path = (char*)gtk_ctree_node_get_row_data (ctree, nd);
	GdkCursor *hand;
	struct stat d_info;
	GtkCTreeNode *tree_sel;	

	if(path==NULL)
	   return;
	   
	switch(flag_cmd) {
		case 0:	
			gtk_ctree_select (ctree, nd);
			strcpy(select_path, path);
			Refresh_file_list(path);
			gtk_entry_set_text(GTK_ENTRY(chdir_edit), select_path);
			gtk_entry_set_text(GTK_ENTRY(fname_edit), "");
			gtk_entry_set_editable(GTK_ENTRY(fname_edit), FALSE);

			/* Affichage des droits sur le repertoire dans la barre d'etats */
			statusbar_display(1, "");
			if(stat(path, &d_info)==-1)
			    perror("stat");
			else {
				char dr[]=" ---------- ";

				stat2strmode(d_info, dr);
				statusbar_display(1, dr);
			}
			break;
		case 1:  
			flag_cmd=0;
			g_print("Copie vers: %s\n", path);
			/* provoque le clignotement du repertoire destination */
			tree_sel=GTK_CTREE_NODE(GTK_CLIST(arbo)->selection->data);
			gtk_ctree_select (ctree, nd);
			while (gtk_events_pending())
			    gtk_main_iteration();
			usleep(10000);
			gtk_ctree_unselect (ctree, nd);
			gtk_clist_set_selection_mode(GTK_CLIST(arbo), GTK_SELECTION_SINGLE);
			gtk_ctree_select (ctree, tree_sel);

/*			cmd_copy_files(path);*/
			gtk_grab_remove(scroll_arbo);
			hand=gdk_cursor_new(GDK_LEFT_PTR);
			gdk_window_set_cursor (mainwin->window, hand);
			break;
		case 2: 
			flag_cmd=0;
			g_print("Dplacer vers: %s\n", path);
			/* provoque le clignotement du repertoire destination */
			tree_sel=GTK_CTREE_NODE(GTK_CLIST(arbo)->selection->data);
			gtk_ctree_select (ctree, nd);
			while (gtk_events_pending())
			    gtk_main_iteration();
			usleep(10000);
			gtk_ctree_unselect (ctree, nd);
			gtk_clist_set_selection_mode(GTK_CLIST(arbo), GTK_SELECTION_SINGLE);
			gtk_ctree_select (ctree, tree_sel);

			cmd_move_files(path);
			gtk_grab_remove(scroll_arbo);
			hand=gdk_cursor_new(GDK_LEFT_PTR);
			gdk_window_set_cursor (mainwin->window, hand);
			break;
	}
}


/********************************************************************/
/* Recherche des sous-repertoires dans un repertoire specifie.      */
/********************************************************************/
int subdir(char *path) {
	DIR *dir;
	struct dirent *file;
	struct stat f_info;
	int ret;
	gchar *current;

	if((current=my_getcwd())==NULL) {
		perror("getcwd");
		exit(1);
	}
	
	if((dir=opendir(path))==NULL) {
		return 2;
	}
	
	chdir(path);
		
	while((file=readdir(dir))!=NULL) {
		if((ret=stat(file->d_name, &f_info))==-1) continue; /* ici on ignore les erreurs */
		else if( S_ISDIR(f_info.st_mode) ) {
			if( strcmp("." , file->d_name)!=0 && strcmp("..", file->d_name)!=0 ) {
				if(closedir(dir)==-1) perror("closedir");
				chdir(current);
				return 1;
			}
		}
	}
	
	if(closedir(dir)==-1) perror("closedir");
	chdir(current);
	if(current!=NULL) free(current);
	return 0;
}


/********************************************************************/
/* Ajoute a un arbre un nouvel element.				    */
/*    name: nom contenu dans le label				    */
/*    path: chemin complet associe a ce repertoire		    */
/*    subdired: contient-il des sous-repertoires		    */
/*    type: repertoire normal(0) ou lien symbolique(1)		    */
/********************************************************************/
void NewDirItem(GtkCTreeNode *parent, char *name, char *path, int subdired, int type) {
	GtkCTreeNode *sibling=NULL;
	char *item_path, *abcd[1];
/*	char link[255];
	int n;*/
	
	
	item_path=(char *)malloc(strlen(name)+strlen(path)+2);
	strcpy(item_path, path);
	if(strcmp(name, "/")!=0 && strcmp(path, "/")!=0 ) strcat(item_path, "/");
	strcat(item_path, name);

	abcd[0]=name;
	if(subdired==2)
		sibling = gtk_ctree_insert_node (GTK_CTREE(arbo), parent, sibling, abcd, 5,
				pix_rdo, msk_rdo, pix_rdo, msk_rdo,
				FALSE, FALSE);
	else {
		if(type)
			sibling = gtk_ctree_insert_node (GTK_CTREE(arbo), parent, sibling, abcd, 5,
				pix_dlnk, msk_dlnk, pix_dlnk, msk_dlnk,
				FALSE, FALSE);
		else 
			sibling = gtk_ctree_insert_node (GTK_CTREE(arbo), parent, sibling, abcd, 5,
				pix_dir, msk_dir, pix_dir, msk_dir,
				FALSE, FALSE);

	}
	gtk_ctree_node_set_row_data_full (GTK_CTREE(arbo), sibling, item_path, (GtkDestroyNotify) arbo_destroy_path);
/*	if(type) {
		n=readlink(item_path, link, 254);
		link[n]='\0';
		gtk_tooltips_set_tip(tips, item, link, NULL);
	}*/

	if(subdired==1) {
		char *pt[1]={"."};
		sibling = gtk_ctree_insert_node (GTK_CTREE(arbo), sibling, NULL, pt, 5,
				NULL, NULL, NULL, NULL,
				TRUE, FALSE);	
	}
}



/********************************************************************/
void CreateBranch(GtkCTreeNode *parent, char *path) {
	struct stat f_info;
	struct dirent *file;
	rep_entry *namelist=NULL, *p, *last, *new, *sav;
	DIR *dir;
	gchar *name, *current;

	if((current=my_getcwd())==NULL) {
		perror("getcwd");
		exit(1);
	}

	if((dir=opendir(path))==NULL) {
		g_print("(%s) ", path);
		perror("opendir");
		return;
	}
	chdir(path);
	
	while((file=readdir(dir))!=NULL) {
		if( strcmp("." , file->d_name)!=0 && strcmp("..", file->d_name)!=0 ) {
			if(stat(file->d_name, &f_info)==-1) continue; /* on ignore car ca doit etre un lien non valide */
			else {
				if( S_ISDIR(f_info.st_mode) ) {
					if(lstat(file->d_name, &f_info)==-1) perror("lstat");
					else {
						name=(gchar*) malloc((strlen(file->d_name)+1)*sizeof(char));
						strcpy(name, file->d_name);
						if(namelist==NULL) { 
							namelist=(rep_entry *)malloc(sizeof(rep_entry));
							namelist->name=name;
							namelist->mode=S_ISLNK(f_info.st_mode);
							namelist->suiv=NULL;
							}
						else {
							p=namelist;
							while(p!=NULL) {
								if(strcmp(name, p->name)<0) break;
								last=p;
								p=p->suiv;
							}
							new=(rep_entry *)malloc(sizeof(rep_entry));
							new->suiv=p;
							new->name=name;
							new->mode=S_ISLNK(f_info.st_mode);
							if(p==namelist) namelist=new;
							else 		last->suiv=new;
						}
					}
				}
			}
		}
	}
	p=namelist;
	while(p!=NULL) {
		NewDirItem(parent, p->name, path, subdir(p->name), p->mode);
		p=p->suiv;
	}
	p=namelist;
	while(p!=NULL) {
		free(p->name);
		sav=p;
		p=p->suiv;
		free(sav);
	}	

	if(closedir(dir)==-1) perror("closedir");
	chdir(current);
	if(current!=NULL) free(current);
}

/***************************************************************************************/
/***************************************************************************************/
/***************************************************************************************/
/***************************************************************************************/

void split_path(char *str, char *(*tb[])) {
	char buff[100];
	int i=0, j, n=0;
	
	while(str[i++]!='\0') {
		j=0;
		while(str[i]!='/' && str[i]!='\0') buff[j++]=str[i++];
		buff[j]='\0';
		if(*tb==NULL) {
			if((*tb=(char**)malloc(sizeof(char*)))==NULL) perror("malloc");
		}
		else {
			if((*tb=(char**)realloc(*tb, (n+1)*sizeof(char*)))==NULL) perror("realloc");
		}
		(*tb)[n++]=strdup(buff);
	}
	*tb=(char**)realloc(*tb, (n+1)*sizeof(char*));
	(*tb)[n]=NULL;
}

void arbo_change_dir(char *path) {
	arbo_change_dir_recursive(path, HOMEnode);
}

static void arbo_change_dir_recursive(char *path, GtkCTreeNode *nd) {
	char *cur=(char *)gtk_ctree_node_get_row_data (GTK_CTREE(arbo), nd);
	
	if(strcmp(cur, path)==0)
		arbo_selected(GTK_CTREE(arbo), nd);
	else {
		/* on verifie si on peut ouvrir la branche */
		if(GTK_CTREE_ROW(nd)->children==NULL) {
			GtkCTreeNode *nod;
			int i=0, j=0;
			char tmp[50], new[255], *row_txt[1];

			while(cur[i]!='\0') i++;
			i++;
			while(path[i]!='\0' && path[i]!='/') tmp[j++]=path[i++];
			tmp[j]='\0';

			sprintf(new, "%s/%s", cur, tmp);
			row_txt[0]=tmp;
			GTK_CTREE_ROW(nd)->expanded = TRUE;
			nod=gtk_ctree_insert_node (GTK_CTREE(arbo), nd, NULL, row_txt, 5,
				pix_dir, msk_dir, pix_dir, msk_dir,
				FALSE, TRUE);
			gtk_ctree_node_set_row_data_full (GTK_CTREE(arbo), nod,
				strdup(new), (GtkDestroyNotify) arbo_destroy_path);

			arbo_change_dir_recursive(path, nod);
		}
		else {
			GList *p;
			/* si on doit ouvrir la branche */
			if(!GTK_CTREE_ROW(nd)->expanded)
				arbo_tree_expand(arbo, nd);

			/* et on regarde si un element de la branche est une partie du chemin recherche */
			p = (GList*) GTK_CTREE_ROW(nd)->children;
			while(p!=NULL) {
				if(IsPartOfPath((char *)gtk_ctree_node_get_row_data(GTK_CTREE(arbo), GTK_CTREE_NODE(p)), path)) {
					arbo_change_dir_recursive(path, GTK_CTREE_NODE(p));
					return;
				}
				p=p->next;
			}
			if(p==NULL)
				printf("%s n'existe pas ou est un fichier\n", path);
		}
	}
}

void arbo_refresh_dir(char *path) {
	arbo_refresh_dir_recursive(path, HOMEnode);
}

static void arbo_refresh_dir_recursive(char *path, GtkCTreeNode *nd) {
	char *cur=(char *)gtk_ctree_node_get_row_data (GTK_CTREE(arbo), nd);
	
	if(strcmp(cur, path)==0) {
	    int sub;
	    sub=subdir(path);
	    if(sub==1) {
		char *pt[1]={"."};
		gtk_ctree_insert_node (GTK_CTREE(arbo), nd, NULL, pt, 5,
				NULL, NULL, NULL, NULL,
				TRUE, FALSE);	
	    }
	    if(sub==0) {
	        gtk_clist_freeze(GTK_CLIST(arbo));
		while(GTK_CTREE_ROW(nd)->children)
		    gtk_ctree_remove_node(GTK_CTREE(arbo), GTK_CTREE_ROW(nd)->children);
	        gtk_clist_thaw(GTK_CLIST(arbo));
	    }
	}
	else {
	    /* on verifie si on peut ouvrir la branche */
	    if(GTK_CTREE_ROW(nd)->children) {
		    GList *p;

		    /* si on doit ouvrir la branche */
		    if(!GTK_CTREE_ROW(nd)->expanded)
			    return;

		    /* et on regarde si un element de la branche est une partie du chemin recherche */
		    p = (GList*) GTK_CTREE_ROW(nd)->children;
		    while(p!=NULL) {
			    if(IsPartOfPath((char *)gtk_ctree_node_get_row_data(GTK_CTREE(arbo), GTK_CTREE_NODE(p)), path)) {
				    arbo_refresh_dir_recursive(path, GTK_CTREE_NODE(p));
				    return;
			    }
			    p=p->next;
		    }
	    }
	}
}

int IsPartOfPath(char *part, char *path) {
	if(path==NULL || part==NULL) return 0;

	while(*path!='\0' && *part!='\0' && *part==*path) {
		part++;
		path++;
	}
	
	if(*part=='\0')
		return 1;
	else
		return 0;
	
}

void stat2strmode(struct stat p, char *s) {
	if(p.st_mode & S_IRUSR) s[2]='r';
	if(p.st_mode & S_IWUSR) s[3]='w';
	if(p.st_mode & S_ISUID) {
		if(p.st_mode & S_IXUSR) s[4]='s';
		else			s[4]='S';
	}
	else if(p.st_mode & S_IXUSR) s[4]='x';

	if(p.st_mode & S_IRGRP) s[5]='r';
	if(p.st_mode & S_IWGRP) s[6]='w';
	if(p.st_mode & S_ISGID) {
		if(p.st_mode & S_IXGRP) s[7]='s';
		else			s[7]='S';
	}
	else if(p.st_mode & S_IXGRP) s[7]='x';

	if(p.st_mode & S_IROTH) s[8]='r';
	if(p.st_mode & S_IWOTH) s[9]='w';
	if(p.st_mode & S_ISVTX) {
		if(p.st_mode & S_IXOTH) s[10]='t';
		else			s[10]='T';
	}
	else if(p.st_mode & S_IXOTH) s[10]='x';
	
	if(S_ISLNK(p.st_mode)) {
		s[1]='l';
		return;
	}

	switch(S_IFMT & p.st_mode)  {
		case S_IFBLK:	s[1]='b';
				break;
		case S_IFDIR:	s[1]='d';
				break;
		case S_IFCHR:	s[1]='c';
				break;
		case S_IFIFO:	s[1]='p';
				break;
		case S_IFSOCK:	s[1]='s';
				break;
	}
}

