/* Copyright (C) Emmanuel Varagnat 1998

   Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
   le modifier conformment aux dispositions de la Licence Publique Gnrale
   GNU, telle que publie par la Free Software Foundation ; version 2 de la
   licence, ou encore ( votre choix) toute version ultrieure.

   Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE
   GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou
   D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence
   Publique Gnrale GNU .

   Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en
   mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
   Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.

  ==========================================================================

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ==========================================================================

   Ecrit par Emmanuel Varagnat (emmanuel.varagnat@altavista.net)  */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <sys/stat.h>

#include "types.h"
#include "config.h"
#include "explore.h"

#define MAX_LANG 1
#define LG_BUFF 255	/* Longueur max d'une ligne */


/* =============================================================== */

struct option opt={	0,	/* langue */
			1,	/* affiche fichiers */
			1,	/* affiche repertoires */
			317,	/* masque colonne */
			1, 0,	/* tri et ordre de tri */
			0, NULL /* tableau des types (taille et pointeur) */
		  };

char *HOMEpath = NULL;
int exec_pos=0;
struct exec_item *exec_tb=NULL;

static char *icon_path = NULL;
static char rcfile[LG_BUFF] = "";

static int analyse(char *str);
static int next_word(char*, int, char*, int);
static void expand_path(char*);

/* =============================================================== */


void load_config() {
	char buff[LG_BUFF], ch;
	int f, nb, pos, lgn=1;
	
	if( !(HOMEpath=(char *)getenv("HOME")) ) {
		fprintf(stderr, "Can't get $HOME variable\n");
		exit(1);
	}

	strcpy(rcfile, HOMEpath);
	strcat(rcfile, "/.gtkrc");
	gtk_rc_parse(rcfile);

	strcpy(rcfile, HOMEpath);
	strcat(rcfile, "/.gxplorc");

	if( (f=open(rcfile, O_RDONLY))==-1 ) {
		if(errno==ENOENT) return;
		else {
			perror("open");
			exit(1);
		}
	}

	do {
		buff[0]='\0';
		pos=0;
		do {
			if( (nb=read(f, &ch, 1))==-1 ) {
				perror("read");
				exit(1);
			}
			if(nb!=0) {
				if(ch!='\n')  {
					if(pos<LG_BUFF-1) buff[pos++]=ch;
					else printf(".gbiffrc: Line %d too long (>128)!!!\n", lgn);
					}
				else {
					buff[pos]='\0';
					break;
				}
			}
		} while(nb!=0);
		if(nb!=0) {
			if( analyse(buff)==-1 )
				printf("Error in .gxplor (l.%d): %s\n", lgn, buff);
		}
		lgn++;
	} while(nb!=0);
	close(f);
}

/* =============================================================== */

static int analyse(char *str) {
	char buff[LG_BUFF];
	int pos=0;
	pos=next_word(str, pos, buff, LG_BUFF);

	if(pos==0) return 0;
	if(buff[0]=='#') return 0;
	g_strup(buff);

	if(strcmp(buff, "ICONPATH")==0) {
		char path[LG_BUFF];
		
		pos=next_word(str, pos, path, LG_BUFF);
		icon_path = strdup(path);
		return 0;
	}
	
	if(strcmp(buff, "ICON")==0) {
		char type[20];
		char path[LG_BUFF], execp[LG_BUFF];
		
		pos=next_word(str, pos, type, 10);
		pos=next_word(str, pos, path, LG_BUFF);
		pos=next_word(str, pos, execp, LG_BUFF);
		g_strup(type);

		opt.nb_types++;
		if(opt.lst_ext==NULL) {
			if((opt.lst_ext=(file_type*)malloc(sizeof(file_type)))==NULL) {
				perror("malloc");
				exit(1);
			}
		}
		else {
			opt.lst_ext=realloc(opt.lst_ext, (opt.nb_types)*sizeof(file_type));
		}

		expand_path(path);
		opt.lst_ext[opt.nb_types-1].ext = strdup(type);
		opt.lst_ext[opt.nb_types-1].path = strdup(path);

		if( (opt.lst_ext[opt.nb_types-1].pix=gdk_pixmap_create_from_xpm(mainwin->window, &opt.lst_ext[opt.nb_types-1].msk,
					&mainwin->style->bg[GTK_STATE_NORMAL], path))==NULL ) {
			g_print("Can't load %s\nLoading default pixmap.\n", path);
			opt.lst_ext[opt.nb_types-1].pix=pix_file;
			opt.lst_ext[opt.nb_types-1].msk=msk_file;
		}

		opt.lst_ext[opt.nb_types-1].exec = strdup(execp);
		return 0;
	}

	if(strcmp(buff, "LANG")==0) {
		char lg[3];
		int lang;
				
		pos=next_word(str, pos, lg, 3);
		sscanf(lg, "%d", &lang);
		if(lang<=MAX_LANG) opt.lang=lang;
		return 0;
	}

	if(strcmp(buff, "EXEC")==0) {
		char m[2], buf[128], **cmd;
		int cmd_pos=0;

		if((pos=next_word(str, pos, m, 2))==-1) return -1;
		if(m[0]=='\0') return -1;

		if((pos=next_word(str, pos, buf, 128))==-1) return -1;
		if(buf[0]=='\0') return -1;

		cmd=(char**)malloc(sizeof(char*));
		cmd[cmd_pos++]=strdup(buf);
		while(str[pos]!='\0') {
			pos=next_word(str, pos, buf, 128);
			if(buf[0]=='\0') break;
			cmd=(char**)realloc(cmd, (cmd_pos+1)*sizeof(char*));
			cmd[cmd_pos++]=strdup(buf);
		}
		cmd=(char**)realloc(cmd, (cmd_pos+1)*sizeof(char*));
		cmd[cmd_pos]=NULL;

		if(exec_tb==NULL)
			exec_tb=(struct exec_item *)malloc(sizeof(struct exec_item));
		else
			exec_tb=(struct exec_item *)realloc(exec_tb, (exec_pos+1)*sizeof(struct exec_item));
		if(strcmp("+", m)==0) exec_tb[exec_pos].multi=1;
		if(strcmp("-", m)==0) exec_tb[exec_pos].multi=0;
		exec_tb[exec_pos].arg=cmd;
		exec_tb[exec_pos].pos=exec_pos;
		exec_pos++;
		
		return 0;
	}

	return -1;
}

/* =============================================================== */

static int next_word(char *line, int pos, char *buff, int max) {
	int i=0;

	buff[0]='\0';
	while(*(line+pos)==' ' || *(line+pos)=='\t') pos++;

	while(*(line+pos)!='\0' && *(line+pos)!=' ' && *(line+pos)!='\t' && i<(max-1)) buff[i++]=*(line+(pos++));

	if(i==(max-1) && *(line+pos)!='\0' && *(line+pos)!=' ' && *(line+pos)!='\t')
	     return -1;

	buff[i]='\0';
	return pos;
}


/* =============================================================== */
/* en fonction de path, remet dans path le chemin absolu */

static void expand_path(char *path) {
	if(path[0]=='/') return;
	if(path[0]=='~') {
		char *tmp;
		
		tmp=(char*) malloc( (strlen(HOMEpath) + strlen(path) )*sizeof(char));
		strcpy(tmp, HOMEpath);
		strcat(tmp, path+1);
		strcpy(path, tmp);
		free(tmp);
	}
	else {
		if(icon_path) {
		    char *tmp;
		
		    tmp=(char*) malloc( (strlen(icon_path) + strlen(path) + 2)*sizeof(char));
		    strcpy(tmp, icon_path);
		    strcat(tmp, "/");
		    strcat(tmp, path);
		    strcpy(path, tmp);
		    free(tmp);
		}
	}
}
