/* Copyright (C) Emmanuel Varagnat 1998

   Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
   le modifier conformment aux dispositions de la Licence Publique Gnrale
   GNU, telle que publie par la Free Software Foundation ; version 2 de la
   licence, ou encore ( votre choix) toute version ultrieure.

   Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE
   GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou
   D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence
   Publique Gnrale GNU .

   Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en
   mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
   Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.

  ==========================================================================

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ==========================================================================

   Ecrit par Emmanuel Varagnat (emmanuel.varagnat@altavista.net)  */


#include <gtk/gtk.h>

#include "types.h"
#include "config.h"


/********************************************************************/
/********************************************************************/
/********************************************************************/
/****   Boite de dialogue pour l'affichage de multiple erreurs  *****/

static GtkWidget *merrorwin=NULL;
static GtkWidget *errlst=NULL;

static char *str_mesg[][NB_LANG] = {
				{"Erreurs", "Errors"}
};

static void close_merror(GtkWidget *widg, gpointer p) {
	gtk_grab_remove(merrorwin);
	gtk_widget_destroy(merrorwin);
	errlst = merrorwin = NULL;
}

void multierror_dialog(char *file, char *msg) {
	char *row[2];

	if(merrorwin == NULL) {
		GtkWidget *vbox;
		GtkWidget *button;
		GtkWidget *sep;

		merrorwin = gtk_window_new( GTK_WINDOW_DIALOG );
		gtk_window_set_title(GTK_WINDOW(merrorwin), str_mesg[0][opt.lang]);
		gtk_widget_set_usize(merrorwin, 380, 190);
		gtk_signal_connect (GTK_OBJECT (merrorwin), "delete_event",
	                               GTK_SIGNAL_FUNC (close_merror), NULL);
		gtk_container_border_width(GTK_CONTAINER(merrorwin), 10);
		gtk_widget_realize(merrorwin);

		vbox=gtk_vbox_new(FALSE, 0);
		gtk_container_add(GTK_CONTAINER(merrorwin), vbox);
		gtk_widget_show(vbox);
	
		errlst=gtk_clist_new(2);
		gtk_clist_set_selection_mode(GTK_CLIST(errlst), GTK_SELECTION_EXTENDED);
		gtk_clist_set_column_width(GTK_CLIST(errlst), 0, 120);
		gtk_box_pack_start(GTK_BOX(vbox), errlst, TRUE, TRUE, 4);
		gtk_widget_show(errlst);

		sep=gtk_hseparator_new();
		gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, TRUE, 4);
		gtk_widget_show(sep);

		button=gtk_button_new_with_label("Ok");
		gtk_signal_connect (GTK_OBJECT (button), "clicked",
	                               GTK_SIGNAL_FUNC (close_merror), NULL);
		gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 4);
		gtk_widget_show(button);

		row[0]=file;
		row[1]=msg;
		gtk_clist_append(GTK_CLIST(errlst), row);

		gtk_grab_add(merrorwin);

		gtk_widget_realize(vbox);
		gtk_widget_realize(sep);
		gtk_widget_realize(errlst);
		gtk_widget_show(merrorwin);
	}
	else {
		gtk_clist_freeze(GTK_CLIST(errlst));
		row[0]=file;
		row[1]=msg;
		gtk_clist_append(GTK_CLIST(errlst), row);
		gtk_clist_thaw(GTK_CLIST(errlst));
	}
	while (gtk_events_pending())
	    gtk_main_iteration();
}




/********************************************************************/
/********************************************************************/
/********************************************************************/
/* Boite de dialogue avec zone d'edition et 2 boutons (Ok/Annuler)  */

static GtkWidget *edit_win;


static void edit_OK(GtkWidget *widget, int (*func)(int)) {

	if(func(TRUE)) {
		gtk_grab_remove(edit_win);
		gtk_widget_destroy(edit_win);
	}
}


static void edit_Annul(GtkWidget *widget, int (*func)(int)) {

	if(func(FALSE)) {
		gtk_grab_remove(edit_win);
		gtk_widget_destroy(edit_win);
	}
}


void edit_dialog(char *text, char *titre, int (*func)(int)) {
	GtkWidget *hbox, *vbox;
	GtkWidget *label, *edit, *b_OK, *b_Annul;
	GtkWidget *sep;
	
	edit_win = gtk_window_new( GTK_WINDOW_DIALOG );
	gtk_window_set_title(GTK_WINDOW(edit_win), titre);
	gtk_window_position( GTK_WINDOW(edit_win), GTK_WIN_POS_CENTER );
	gtk_widget_set_usize(edit_win, 250, 0); 
	gtk_widget_realize(edit_win);
	
	vbox=gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(edit_win), vbox);
	gtk_container_border_width(GTK_CONTAINER(vbox), 10);
	gtk_widget_show(vbox);

	label=gtk_label_new(text);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);
	
	edit=gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(vbox), edit, FALSE, FALSE, 0);
	gtk_widget_show(edit);	

	sep=gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 10);
	gtk_widget_show(sep);

	hbox=gtk_hbox_new(FALSE, 10);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
/*	gtk_container_border_width(GTK_CONTAINER(hbox), 10);*/
	gtk_widget_show(hbox);
	
	b_OK=gtk_button_new_with_label("Ok");	
	gtk_signal_connect (GTK_OBJECT (b_OK), "clicked",
                           GTK_SIGNAL_FUNC (edit_OK), func);
	gtk_box_pack_start(GTK_BOX(hbox), b_OK, FALSE, FALSE, 0);
	gtk_widget_show(b_OK);
	b_Annul=gtk_button_new_with_label("Annuler");	
	gtk_signal_connect (GTK_OBJECT (b_Annul), "clicked",
                           GTK_SIGNAL_FUNC (edit_Annul), func);
	gtk_box_pack_start(GTK_BOX(hbox), b_Annul, FALSE, FALSE, 0);
	gtk_widget_show(b_Annul);
	
	gtk_grab_add(edit_win);
	gtk_widget_show(edit_win);
}




/********************************************************************/
/********************************************************************/
/********************************************************************/
/********   Boite de dialogue simple avec un bouton OK   ************/

static void delete_msgbox(GtkWidget *widget, gpointer p) {
	gtk_grab_remove(widget);
	gtk_widget_destroy(widget);
	gtk_main_quit();
}

static void close_msgbox(GtkWidget *widget, GtkWidget *p) {
	gtk_grab_remove(p);
	gtk_widget_destroy(p);
	gtk_main_quit();
}	

void msg_dialog(char *title, char *text) {
	GtkWidget *msgbox;
	GtkWidget *vbox, *hbox;
	GtkWidget *label, *b_OK, *sep;
	
	msgbox=gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_position(GTK_WINDOW(msgbox), GTK_WIN_POS_CENTER);
	gtk_window_set_title(GTK_WINDOW(msgbox), title);
	gtk_signal_connect (GTK_OBJECT (msgbox), "delete_event",
                               GTK_SIGNAL_FUNC (delete_msgbox), NULL);
	gtk_container_border_width(GTK_CONTAINER(msgbox), 10);
	gtk_grab_add(msgbox);

	vbox=gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(msgbox), vbox);
	gtk_widget_show(vbox);
	
	label=gtk_label_new(text);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	hbox=gtk_hbox_new(FALSE, 5);
	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);
	b_OK=gtk_button_new_with_label("Ok");
	gtk_widget_set_usize(b_OK, 60, 0);
	gtk_signal_connect (GTK_OBJECT (b_OK), "clicked",
                               GTK_SIGNAL_FUNC (close_msgbox), msgbox);
	gtk_box_pack_start(GTK_BOX(hbox), b_OK, FALSE, FALSE, 0);
	gtk_widget_show(b_OK);

	sep=gtk_hseparator_new();
	gtk_box_pack_end(GTK_BOX(vbox), sep, FALSE, FALSE, 5);
	gtk_widget_show(sep);

	gtk_widget_show(msgbox);
	gtk_main();
}

/********************************************************************/
/********************************************************************/
/********************************************************************/
/********  Boite de dialogue avec confirmation (Oui/Non)   **********/
/*
static int yesno_val;

void yesno_ret(GtkWidget *widget, int val) {
	yesno_val=val;
	gtk_main_quit();
}

int yesno_dialog(char *text, char *titre) {
	GtkWidget *hbox, *vbox;
	GtkWidget *label, *b_Oui, *b_Non;
	GtkWidget *sep;
	GtkWidget *win;
	
	win = gtk_window_new( GTK_WINDOW_DIALOG );
	gtk_window_set_title(GTK_WINDOW(win), titre);
	gtk_window_position( GTK_WINDOW(win), GTK_WIN_POS_CENTER );
	gtk_widget_set_usize(win, 250, 0); 
	gtk_widget_realize(win);
	
	vbox=gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(win), vbox);
	gtk_container_border_width(GTK_CONTAINER(vbox), 10);
	gtk_widget_show(vbox);

	label=gtk_label_new(text);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);
	
	sep=gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 10);
	gtk_widget_show(sep);

	hbox=gtk_hbox_new(FALSE, 10);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);
	
	b_Oui=gtk_button_new_with_label("Oui");	
	gtk_signal_connect (GTK_OBJECT (b_Oui), "clicked",
                           GTK_SIGNAL_FUNC (yesno_val), (gpointer)1);
	gtk_box_pack_start(GTK_BOX(hbox), b_Oui, FALSE, FALSE, 0);
	gtk_widget_show(b_Oui);
	b_Non=gtk_button_new_with_label("Non");	
	gtk_signal_connect (GTK_OBJECT (b_Non), "clicked",
                           GTK_SIGNAL_FUNC (yesno_val), (gpointer)0);
	gtk_box_pack_start(GTK_BOX(hbox), b_Non, FALSE, FALSE, 0);
	gtk_widget_show(b_Non);
	
	gtk_grab_add(win);
	gtk_widget_show(win);

	gtk_main();

	gtk_grab_remove(win);
	gtk_widget_destroy(win);
	return yesno_val;
}
*/




/********************************************************************/
/********************************************************************/
/********************************************************************/
/********  Boite de dialogue de confirmation pour fichier  **********/
/********  Boutons :  Oui, Non, Tous, Annuler              **********/
/*
static int val_onta;	

static void return_onta(GtkWidget *widget, int val) {
	val_onta=val;
	gtk_main_quit();
}

int onta_dialog(char *text, char *titre) {
	GtkWidget *hbox, *vbox;
	GtkWidget *label, *b_Oui, *b_Non, *b_Tout, *b_Annul;
	GtkWidget *win;
	GtkWidget *sep;
	
	win = gtk_window_new( GTK_WINDOW_DIALOG );
	gtk_window_set_title(GTK_WINDOW(win), titre);
	gtk_window_position( GTK_WINDOW(win), GTK_WIN_POS_CENTER );
	gtk_widget_set_usize(win, 250, 0); 
	gtk_widget_realize(win);
	
	vbox=gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(win), vbox);
	gtk_container_border_width(GTK_CONTAINER(vbox), 10);
	gtk_widget_show(vbox);

	label=gtk_label_new(text);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);
	
	sep=gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 10);
	gtk_widget_show(sep);

	hbox=gtk_hbox_new(FALSE, 10);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);
	
	b_Oui=gtk_button_new_with_label("Oui");	
	gtk_signal_connect (GTK_OBJECT (b_Oui), "clicked",
                           GTK_SIGNAL_FUNC (return_onta), (gpointer)1);
	gtk_box_pack_start(GTK_BOX(hbox), b_Oui, FALSE, FALSE, 0);
	gtk_widget_show(b_Oui);

	b_Non=gtk_button_new_with_label("Non");	
	gtk_signal_connect (GTK_OBJECT (b_Non), "clicked",
                           GTK_SIGNAL_FUNC (return_onta), (gpointer)2);
	gtk_box_pack_start(GTK_BOX(hbox), b_Non, FALSE, FALSE, 0);
	gtk_widget_show(b_Non);

	b_Tout=gtk_button_new_with_label("Tout");
	gtk_signal_connect (GTK_OBJECT (b_Tout), "clicked",
                           GTK_SIGNAL_FUNC (return_onta), (gpointer)3);
	gtk_box_pack_start(GTK_BOX(hbox), b_Tout, FALSE, FALSE, 0);
	gtk_widget_show(b_Tout);

	b_Annul=gtk_button_new_with_label("Annuler");	
	gtk_signal_connect (GTK_OBJECT (b_Annul), "clicked",
                           GTK_SIGNAL_FUNC (return_onta), (gpointer)4);
	gtk_box_pack_start(GTK_BOX(hbox), b_Annul, FALSE, FALSE, 0);
	gtk_widget_show(b_Annul);
	
	gtk_grab_add(win);
	gtk_widget_show(win);

	gtk_main();

	gtk_grab_remove(win);
	gtk_widget_destroy(win);

	return val_onta;
}

*/


/********************************************************************/
/********************************************************************/
/********************************************************************/
/***************  Boite de dialogue avec N boutons   ****************/

static int button_val;

static void button_ret(GtkWidget *widget, int val) {
	button_val=val;
	gtk_main_quit();
}

int buttons_dialog(char *text, char *titre, char *labels[]) {
	GtkWidget *hbox, *vbox;
	GtkWidget *label, *but;
	GtkWidget *sep;
	GtkWidget *win;
	int val;
	
	win = gtk_window_new( GTK_WINDOW_DIALOG );
	gtk_window_set_title(GTK_WINDOW(win), titre);
	gtk_window_position( GTK_WINDOW(win), GTK_WIN_POS_CENTER );
	gtk_widget_set_usize(win, 300, 0); 

	vbox=gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(win), vbox);
	gtk_container_border_width(GTK_CONTAINER(vbox), 10);
	gtk_widget_show(vbox);

	label=gtk_label_new(text);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_label_set_line_wrap (GTK_LABEL(label), TRUE);
	gtk_widget_show(label);
	
	sep=gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 10);
	gtk_widget_show(sep);

	hbox=gtk_hbox_new(FALSE, 10);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
/*	gtk_container_border_width(GTK_CONTAINER(hbox), 10);*/
	gtk_widget_show(hbox);
	
	val=0;
	while(labels[val]!=NULL) {
	    but=gtk_button_new_with_label(labels[val]);
	    gtk_signal_connect (GTK_OBJECT (but), "clicked",
                               GTK_SIGNAL_FUNC (button_ret), (gpointer)val++);
	    gtk_box_pack_start(GTK_BOX(hbox), but, FALSE, FALSE, 0);
	    gtk_widget_show(but);
	}
	
	gtk_grab_add(win);

	gtk_widget_show(win);

	gtk_main();

	gtk_grab_remove(win);
	gtk_widget_destroy(win);
	return button_val;
}
