/* Copyright (C) Emmanuel Varagnat 1998

   Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
   le modifier conformment aux dispositions de la Licence Publique Gnrale
   GNU, telle que publie par la Free Software Foundation ; version 2 de la
   licence, ou encore ( votre choix) toute version ultrieure.

   Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE
   GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou
   D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence
   Publique Gnrale GNU .

   Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en
   mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
   Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.

  ==========================================================================

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ==========================================================================

   Ecrit par Emmanuel Varagnat (emmanuel.varagnat@altavista.net)  */

#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <unistd.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <signal.h>
#include <time.h>
#include <pwd.h>
#include <grp.h>
#include <errno.h>

#include "types.h"
#include "arbo.h"
#include "config.h"
#include "explore.h"
#include "files.h"
#include "popups.h"

#define SUP_OU_INF(b, c1, c2) (b)? (c1>=c2) : (c1<=c2)

typedef struct colonne {
	int width;
	GtkJustification justify;
} Colonne;

typedef struct fich {
			gchar *name, *txt;
			int isdir;
			struct stat info;
			struct fich *suiv;
			int ico;		/* indice dans le tableau des icones */
			GdkPixmap *pix;
			GdkBitmap *msk;
	} fich_entry;

typedef struct user {
			uid_t uid;
			char *name;
			struct user *next;
	} useruid;

typedef struct grpnid {
			gid_t gid;
			char *name;
			struct grpnid *next;
	} grpnid;


/* ---------------Drag & Drop-------------- */

enum {
  TARGET_STRING,
  TARGET_ROOTWIN
};

static GtkTargetEntry target_table[] = {
  { "STRING",     0, TARGET_STRING },
  { "text/plain", 0, TARGET_STRING },
  { "application/x-rootwin-drop", 0, TARGET_ROOTWIN }
};

static guint n_targets = sizeof(target_table) / sizeof(target_table[0]);

/* ---------------Drag & Drop-------------- */

static char *titre_colonne[][NB_LANG] = {	
				{ "nom", 	"name"},
				{ "inoeud",	"inode"},
				{ "droits", 	"permissions"},
				{ "taille",	"size"},
				{ "UID",	"UID"},
				{ "GID",	"GID"},
				{ "liens",	"links"},
				{ "blocs",	"blocks"},
				{ "cration",	"creation"},
				{ "acces",	"access"},
				{ "modification", "modification"}
};

static struct colonne col_define[] = {
				{ 160, GTK_JUSTIFY_LEFT },
				{ 60,  GTK_JUSTIFY_RIGHT },
				{ 80,  GTK_JUSTIFY_LEFT },
				{ 60,  GTK_JUSTIFY_RIGHT },
				{ 50,  GTK_JUSTIFY_CENTER },
				{ 50,  GTK_JUSTIFY_CENTER },
				{ 35,  GTK_JUSTIFY_RIGHT },
				{ 35,  GTK_JUSTIFY_RIGHT },
				{ 170, GTK_JUSTIFY_LEFT },
				{ 170, GTK_JUSTIFY_LEFT },
				{ 170, GTK_JUSTIFY_LEFT },
};

static struct user *userlst=NULL;
static struct grpnid *grouplst=NULL;
static int UID, GID;

GtkWidget *filelst;

static void filelst_drag_data_get();
static void filelst_drag_data_delete();
static void filelst_drag_begin();
static void filelist_motion(GtkWidget*, GdkEventMotion*);
static void button_release(GtkWidget*, GdkEventButton*);
static void button_press(GtkWidget*, GdkEventButton*);
static void free_file_data(File_data*);
static void filelist_column(GtkWidget*, gint);
static void Set_chname(void);
static int  get_xpm_file_type(char*, struct stat, struct stat, GdkPixmap **, GdkBitmap**);
static void sorted_alloc_file_entry(fich_entry**, char*, char*,struct stat, int, GdkPixmap*, GdkBitmap*, int);
static char *getusrname(uid_t);
static char *getgrpname(gid_t);

/******************************************************************/
/******************************************************************/
/******************************************************************/

GtkWidget *InitFileList(void) {
	int colmsk=opt.colonne, msk=1, n=0, i;
	struct stat HOMEinfo;
	
	stat(HOMEpath, &HOMEinfo);
	UID = HOMEinfo.st_uid;
	GID = HOMEinfo.st_gid;
		
	/* calcul le nombre de colonne a partir du masque */
	while(colmsk!=0) {
		if(colmsk & msk) {
			colmsk &= ~msk;
			n++;
		}
		msk=msk << 1;
	}

	filelst=gtk_clist_new(n);
	gtk_widget_set_name(filelst, "filelist");
	gtk_clist_set_row_height(GTK_CLIST(filelst), 18);
	gtk_clist_column_titles_show(GTK_CLIST(filelst));
	gtk_signal_connect(GTK_OBJECT (filelst), "button_release_event",
                               GTK_SIGNAL_FUNC (button_release), NULL);
	gtk_signal_connect(GTK_OBJECT (filelst), "button_press_event",
                               GTK_SIGNAL_FUNC (button_press), NULL);
	gtk_signal_connect(GTK_OBJECT(filelst), "click_column",
				GTK_SIGNAL_FUNC(filelist_column), NULL);
	gtk_signal_connect(GTK_OBJECT(filelst), "motion_notify_event", 
				GTK_SIGNAL_FUNC(filelist_motion), NULL);
	/* Drag & Drop */
	gtk_drag_source_set (filelst, GDK_BUTTON1_MASK,
		       target_table, n_targets, 
		       GDK_ACTION_COPY | GDK_ACTION_MOVE);
	gtk_signal_connect (GTK_OBJECT (filelst), "drag_begin",
			    GTK_SIGNAL_FUNC (filelst_drag_begin), NULL);
	gtk_signal_connect (GTK_OBJECT (filelst), "drag_data_get",
			    GTK_SIGNAL_FUNC (filelst_drag_data_get), NULL);
	gtk_signal_connect (GTK_OBJECT (filelst), "drag_data_delete",
			    GTK_SIGNAL_FUNC (filelst_drag_data_delete), NULL);
	
	msk=1;	n=0; i=0;
	colmsk=opt.colonne;
	while(colmsk!=0) {
		if(colmsk & msk) {
			gtk_clist_set_column_title( GTK_CLIST(filelst), n, 
                                       				titre_colonne[i][opt.lang]);
			gtk_clist_set_column_width(GTK_CLIST(filelst), n, col_define[i].width);
			gtk_clist_set_column_justification(GTK_CLIST(filelst), n,
									col_define[i].justify);
			colmsk &= ~msk;
			n++;
		}
		msk=msk << 1;
		i++;
	}

	return filelst;
}


/******************************************************************/
/******************************************************************/

static void filelst_drag_data_get  (GtkWidget          *widget,
		       GdkDragContext     *context,
		       GtkSelectionData   *selection_data,
		       guint               info,
		       guint               time,
		       gpointer            data)
{
  if (info == TARGET_ROOTWIN)
    g_print ("I was dropped on the rootwin\n");
  else {
    gtk_selection_data_set (selection_data,
			    selection_data->target,
			    8, "I'm Data!", 9);
  }
}

static void filelst_drag_data_delete  (GtkWidget          *widget,
				       GdkDragContext     *context,
				       gpointer            data)
{
  g_print ("Delete the data!\n");
}

static void filelst_drag_begin (GtkWidget      *widget,
		   GdkDragContext *context,
		   GtkWidget *mainwin)
{
	save_row=GTK_CTREE_NODE(GTK_CLIST(arbo)->selection->data);
}

static void timeout_handler_motion(gpointer p) {
	static int n = 0;
	g_print("%d\n", n++);
}

static gint passive_func(GtkWidget *widget, GdkEvent *ev) {
	if(ev->type==GDK_BUTTON_RELEASE)
		write(1, "%", 1);
	return FALSE;
}

static int timer_id=0;
static int bt_4_select;

static void filelist_motion(GtkWidget *widget, GdkEventMotion *event) {	
	static int lgn = -1;
	static int sens = 0;
	static int old_pos = 0;
	int row, col;

	/* on ignore les cliques sur les barres de defilement */
	if(event->window != GTK_CLIST(filelst)->clist_window) {
	    if(event->window==GTK_CLIST(filelst)->title_window && bt_4_select==2) {
/*   	        gtk_clist_get_selection_info(GTK_CLIST(filelst), event->x, event->y, &row, &col);
		gtk_clist_moveto(GTK_CLIST(filelst), row-2, -1, 0, 0); */
		if(!timer_id) {
		   timer_id=gtk_timeout_add(100, (GtkFunction)timeout_handler_motion, NULL); 
		}
		return;
	    }
	    return;
	}

	if(bt_4_select==2) {
	    gtk_signal_connect (GTK_OBJECT (filelst), "event",
				(GtkSignalFunc) passive_func, NULL);
	    gtk_clist_get_selection_info(GTK_CLIST(filelst), event->x, event->y, &row, &col);
	    if(lgn!=row) {
		if(g_list_find(GTK_CLIST(filelst)->selection, (gpointer)row)) {
		    gtk_clist_unselect_row(GTK_CLIST(filelst), row, col);
		    Set_chname();
		}
		else {
		    gtk_clist_set_selection_mode(GTK_CLIST(filelst), GTK_SELECTION_MULTIPLE);
		    gtk_clist_select_row(GTK_CLIST(filelst), row, col);
		    Set_chname();
		}
		lgn=row;
	    }
	    else {
	       int d;
	       d=event->y - old_pos;
	       if((sens>=0 && d<0) || (sens<=0 && d>0)) {
		  if(g_list_find(GTK_CLIST(filelst)->selection, (gpointer)row)) {
		      gtk_clist_unselect_row(GTK_CLIST(filelst), row, col);
		      Set_chname();
		  }
		  else {
		      gtk_clist_set_selection_mode(GTK_CLIST(filelst), GTK_SELECTION_MULTIPLE);
		      gtk_clist_select_row(GTK_CLIST(filelst), row, col);
		      Set_chname();
		  }
	       }
	       sens=d;
	    }
	    old_pos = event->y;
	}
}

static void button_release(GtkWidget *widget, GdkEventButton *event) {
	int row, col;

	/* on s'occupe du timer */
	if(bt_4_select==2 && timer_id) {
		gtk_timeout_remove(timer_id);
		timer_id=0;
		return;
	}

	/* on ignore les cliques sur les barres de defilement */
	if(event->window != GTK_CLIST(filelst)->clist_window)
		return;

	if(event->button==1) {
		gtk_clist_get_selection_info(GTK_CLIST(filelst), event->x, event->y, &row, &col);
		gtk_clist_set_selection_mode(GTK_CLIST(filelst), GTK_SELECTION_SINGLE);
		gtk_clist_unselect_all(GTK_CLIST(filelst));
		gtk_clist_select_row(GTK_CLIST(filelst), row, col);
		Set_chname();
	}
}

static void button_press(GtkWidget *widget, GdkEventButton *event) {
	int row, col;

	/* on ignore les cliques sur les barres de defilement */
	if(event->window != GTK_CLIST(filelst)->clist_window)
		return;

	if (event->type == GDK_2BUTTON_PRESS) {
		if(event->button==1) {
			File_data *p;
			GtkCTreeNode *child;
			GtkCTreeNode *tree_sel;
			char *txt;
			guint8 spc;
			GdkBitmap *btmp;
			GdkPixmap *pxmp;
			
			if(GTK_CLIST(arbo)->selection==NULL)
			    arbo_change_dir(select_path);
			
			tree_sel=GTK_CTREE_NODE(GTK_CLIST(arbo)->selection->data);
/*			g_list_free(GTK_CLIST(filelst)->selection);
			GTK_CLIST(filelst)->selection=NULL;*/
			
			gtk_clist_get_selection_info(GTK_CLIST(filelst), event->x, event->y, &row, &col);
			p=(File_data*)gtk_clist_get_row_data(GTK_CLIST(filelst), row);

			if(p->type==0) {
			    /* si on appuye sur la touche shift lors du double-clique */
			    /* cela permet de d'ajouter des options  l'application lance */
			    printf("Special %x\n", event->state);
			    if(p->ext>0)
			    	printf("%s\n", opt.lst_ext[p->ext].exec);
			    if(p->ext==-2)
			    	printf("Executer\n");
			    return;
			}

			if(!GTK_CTREE_ROW(tree_sel)->expanded)
				arbo_tree_expand(arbo, tree_sel);

			/* recherche l'element de meme nom dans le sous arbre */			
			child=GTK_CTREE_ROW(tree_sel)->children;
			while(child != NULL) {
				if(gtk_ctree_node_get_pixtext(GTK_CTREE(arbo), child, 0, &txt, &spc, &pxmp, &btmp)
				    && strcmp(txt, p->nom)==0) {
					char *path = (char *)gtk_ctree_node_get_row_data(GTK_CTREE(arbo), child);

					gtk_ctree_select(GTK_CTREE(arbo), child);
					strcpy(select_path, path);
					Refresh_file_list(path);
					return;
				}
				child = GTK_CTREE_NODE_NEXT(child);
			}
		}
		return;
	}

	if (event->type == GDK_BUTTON_PRESS) {
		/* methode bourrin pour empecher qu'un double-clique ne genere
		un signal GDK_BUTTON_PRESS de plus */
		GdkEvent *ev=gdk_event_peek();
		
		if(ev && ev->type==GDK_2BUTTON_PRESS)
			return;

		if(gtk_clist_get_selection_info(GTK_CLIST(filelst),
					event->x, event->y, &row, &col)==0) {
			return;
		}

		bt_4_select=event->button;
		if(event->button==3) {
			GList *p;
			
			p=GTK_CLIST(filelst)->selection;

			/* si rien n'est selectionne affiche le menu et selectionne l'element */
			/* sinon on regarde si le pointeur de la souris est sur une selection */
			/* et si oui on affiche le menu					*/
			if(p==NULL) {
				gtk_clist_select_row(GTK_CLIST(filelst), row, col);
				gtk_menu_popup (GTK_MENU(pop_file), NULL, NULL, NULL, NULL,
						event->button, event->time);
				Set_chname();
				}
			else {
				if(g_list_find(p, (gpointer)row)!=0)
					gtk_menu_popup (GTK_MENU(pop_file), NULL, NULL, NULL, NULL,
						event->button, event->time);
			}
		}
		else if(event->button==2) {
			GList *p=GTK_CLIST(filelst)->selection;
			
			gtk_clist_set_selection_mode(GTK_CLIST(filelst), GTK_SELECTION_MULTIPLE);
			while(p) {
				if((int)p->data==row) {
					gtk_clist_unselect_row(GTK_CLIST(filelst), row, col);
					Set_chname();
					return;
				}
				p=p->next;
			}
			gtk_clist_select_row(GTK_CLIST(filelst), row, col);
			Set_chname();
			
			return;
		}
		else if(event->button==1) {
/*			gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button_press_event");*/
			gtk_clist_set_selection_mode(GTK_CLIST(filelst), GTK_SELECTION_MULTIPLE);
/*			gtk_clist_set_selectable (GTK_CLIST(filelst), row, FALSE);*/
/*			gtk_clist_select_row(GTK_CLIST(filelst), row, col);

			Set_chname();
			return;*/
		}
	}
}


static void Set_chname(void) {
	GList *p=GTK_CLIST(filelst)->selection;

	if(g_list_length(p)==1) {
		File_data *f=(File_data*)gtk_clist_get_row_data(GTK_CLIST(filelst), (int)p->data);
		gtk_entry_set_editable(GTK_ENTRY(fname_edit), TRUE);
		gtk_entry_set_text(GTK_ENTRY(fname_edit), f->nom);
	}
	else {
		gtk_entry_set_editable(GTK_ENTRY(fname_edit), FALSE);
		gtk_entry_set_text(GTK_ENTRY(fname_edit), "");
	}
}

/******************************************************************/
/******************************************************************/
/******************************************************************/


static void free_file_data(File_data *p) {
	free(p->nom);
	free(p);
}


static void filelist_column(GtkWidget *wid, gint col) {
	int msk=1;

	while(col>=0) {
		if(opt.colonne & msk) col--;
		msk = msk << 1;
	}

	msk = msk >> 1;
	if(opt.tri == msk )
		opt.tri_inv = opt.tri_inv ? 0 : 1 ;
	else {
		opt.tri = msk;
		opt.tri_inv = 0;
	}
	Refresh_file_list(select_path);
}


/* Renvoi les pointeurs sur le pixmap et le mask en fonction de    */
/* l'extension du fichier 					   */
/* Renvoi un < 0 pour les types dfinis en dur (defaut, executable...) */
static int get_xpm_file_type(char *nom, struct stat finfo, struct stat linfo, GdkPixmap **pix, GdkBitmap **msk) {
	int sz=strlen(nom), i;
	char ext[10], *dyn_ext=NULL, *p;
	
	*pix=pix_file;
	*msk=msk_file;

	/* Test si c'est un repertoire */
	if(S_ISDIR(finfo.st_mode)) {
		if( (finfo.st_mode & (S_IROTH|S_IXOTH))!=0
		     || ((finfo.st_mode & (S_IRGRP|S_IXGRP))!=0 && finfo.st_gid==GID)
		     || ((finfo.st_mode & (S_IRUSR|S_IXUSR))!=0 && finfo.st_uid==UID)) {
			*pix=pix_dir;
			*msk=msk_dir;
			return -1;
		}
		else {
			*pix=pix_rdo;
			*msk=msk_rdo;
			return -1;
		}
	}

	/* Test si c'est un Makefile */
	if(strcmp("Makefile", nom)==0) {
		*pix=pix_make;
		*msk=msk_make;
		return -1;
	}

	/* Test si c'est une librairie */
	i=0;
	while(nom[i]!='\0' && nom[i]!='.') i++;
	i++;
	if(nom[i]=='a' && (nom[i+1]=='\0' || nom[i+1]=='.')) {
		*pix=pix_a;
		*msk=msk_a;
		return -1;	
	}
	if(nom[i]=='o' && (nom[i+1]=='\0' || nom[i+1]=='.')) {
		*pix=pix_o;
		*msk=msk_o;
		return -1;	
	}
	if(nom[i]=='s' && nom[i+1]=='o' && (nom[i+2]=='\0' || nom[i+2]=='.')) {
		*pix=pix_o;
		*msk=msk_o;
		return -1;	
	}

	i=sz;
	while(i>=0 && nom[i]!='.') i--;
	if(i!=-1) {
		if((sz-i)<10) {
			strcpy(ext, (char *)(nom+i+1));
			p=ext;
		}
		else {
			dyn_ext=(char*) malloc((sz+1)*sizeof(char));
			strcpy(dyn_ext, (char *)(nom+i+1));
			p=dyn_ext;
		}

		if(strcmp("a", p)==0) {
			*pix=pix_a;
			*msk=msk_a;
			return -1;	
		}
		if(strcmp("so", p)==0 || strcmp("o", p)==0) {
			*pix=pix_o;
			*msk=msk_o;
			return -1;	
		}
			
		g_strup(p);
		i=0;
		while(i<opt.nb_types && opt.lst_ext!=NULL) {
			if(strcmp(opt.lst_ext[i].ext, p)==0) {
				*pix=opt.lst_ext[i].pix;
				*msk=opt.lst_ext[i].msk;
				if(dyn_ext!=NULL) free(dyn_ext);
				return i;
			}
			i++;
		}
	}

	/* Test si le fichier est un lien et si il est executable */
	if(S_ISLNK(linfo.st_mode)) {
		if((finfo.st_mode & S_IEXEC)==S_IEXEC) {
			*pix=pix_exe;
			*msk=msk_exe;
			return -2;
		}
		else {
			*pix=pix_flnk;
			*msk=msk_flnk;
			return -1;
		}
	}
	else if((linfo.st_mode & S_IEXEC)==S_IEXEC) {
			*pix=pix_exe;
			*msk=msk_exe;
			return -2;
	     }

	if(dyn_ext!=NULL)
	    free(dyn_ext);
	return -1;
}


void Refresh_file_list(char *path) {
	GdkPixmap *pix;
	GdkBitmap *msk;
	DIR *dir;
	struct dirent *file;
	struct stat f_info, l_info;
	fich_entry *namelist=NULL, *p, *sav;
	gchar info_dir[50], *current=NULL;
	int nb_file=0;
	long nb_byte=0;

	/* Actualise la liste des fichiers avec le nouveau repertoire */
	gtk_clist_clear (GTK_CLIST (filelst));

	if((current=my_getcwd())==NULL) {
		perror("getcwd");
		exit(1);
	}

	if((dir=opendir(path))==NULL) return;

	chdir(path);

	/* recupere les fichiers et les tris */
	gtk_clist_freeze (GTK_CLIST (filelst));
	while((file=readdir(dir))!=NULL) {
		if(strcmp(file->d_name, ".")==0 || strcmp(file->d_name, "..")==0) continue;
		if(stat(file->d_name, &f_info)==-1) {
			char str[128];
			sprintf(str, "stat: (%s)", file->d_name);
			perror(str);
			if(lstat(file->d_name, &l_info)==-1) perror("lstat");
			else {
				char link[255]="";

				if(S_ISLNK(l_info.st_mode)) {
					int n;
					n=readlink(file->d_name, link, 254);
					link[n]='\0';
				}
		
				f_info.st_mode=0;	/* pour eviter toute erreur :-) */
				sorted_alloc_file_entry(&namelist, file->d_name, link, l_info, 0, pix, msk,
					get_xpm_file_type(file->d_name, f_info, l_info, &pix, &msk));
			}
		}
		else {
			if(S_ISDIR(f_info.st_mode)) {
				if(lstat(file->d_name, &l_info)==-1) perror("lstat");
				else {
					char link[255]="";

					if(S_ISLNK(l_info.st_mode)) {
						int n;
						n=readlink(file->d_name, link, 254);
						link[n]='\0';
					}
					
					sorted_alloc_file_entry(&namelist, file->d_name, link, l_info, 1, pix, msk,
						get_xpm_file_type(file->d_name, f_info, l_info, &pix, &msk));
				}
			}
			else {
				if(lstat(file->d_name, &l_info)==-1)
					perror("lstat");
				else {
					char link[255]="";
					if(S_ISLNK(l_info.st_mode)) {
						int n;
						n=readlink(file->d_name, link, 254);
						link[n]='\0';
					}
					
					sorted_alloc_file_entry(&namelist, file->d_name, link, l_info, 0, pix, msk,
						get_xpm_file_type(file->d_name, f_info, l_info, &pix, &msk));
				}
			}
		}
	}

	/* actualise la liste */
	p=namelist;
	while(p!=NULL) {
		File_data *dat;
		char *ret, *textlist[12];
		char dr[]=" ---------- ";
		char s_inod[10], s_size[15], s_link[5], s_bloc[5];	
		char s_ctime[50], s_atime[50], s_mtime[50];
		int row, col=1;

		textlist[0]=NULL;
		if(opt.colonne & AFF_INOD) {
			sprintf(s_inod, "%ld", (p->info).st_ino);
			textlist[col++]=s_inod;
		}
		if(opt.colonne & AFF_PERM) {
			stat2strmode(p->info, dr);
			textlist[col++]=dr;
		}
		if(opt.colonne & AFF_SIZE) {
			sprintf(s_size, "%ld", (p->info).st_size);
			textlist[col++]=s_size;
		}
		if(opt.colonne & AFF_UID) {
			textlist[col++]=getusrname((p->info).st_uid);
		}
		if(opt.colonne & AFF_GID) {
			textlist[col++]=getgrpname((p->info).st_gid);
		}
		if(opt.colonne & AFF_LNK) {
			sprintf(s_link, "%ld", (long)(p->info).st_nlink);
			textlist[col++]=s_link;
		}
		if(opt.colonne & AFF_BLOC) {
			sprintf(s_bloc, "%ld", (long)(p->info).st_blocks);
			textlist[col++]=s_bloc;
		}
		if(opt.colonne & AFF_CTIME) {
			ret=ctime(&(p->info).st_ctime);
			memccpy(s_ctime, ret, '\0', 49);
			textlist[col++]=s_ctime;
		}
		if(opt.colonne & AFF_ATIME) {
			ret=ctime(&(p->info).st_atime);
			memccpy(s_atime, ret, '\0', 49);
			textlist[col++]=s_atime;
		}
		if(opt.colonne & AFF_MTIME) {
			ret=ctime(&(p->info).st_mtime);
			memccpy(s_mtime, ret, '\0', 49);
			textlist[col++]=s_mtime;
		}

		row=gtk_clist_append(GTK_CLIST(filelst), textlist);
		gtk_clist_set_pixtext(GTK_CLIST (filelst), row, 0, p->txt, 5, p->pix, p->msk);
		
		dat=(File_data*) malloc(sizeof(File_data));
		dat->nom = strdup(p->name);
		dat->type = S_ISDIR((p->info).st_mode);
		dat->ext = p->ico;
		gtk_clist_set_row_data_full(GTK_CLIST(filelst), row, dat, (GtkDestroyNotify)free_file_data);

		nb_file++;
		nb_byte+=p->info.st_size;
		p=p->suiv;
	}
	sprintf(info_dir, "%d fichiers, %ld octets", nb_file, nb_byte);
	statusbar_display(2, info_dir);

	/* desalloue la liste chainee */
	p=namelist;
	while(p!=NULL) {
		free(p->name);
		free(p->txt);
		sav=p;
		p=p->suiv;
		free(sav);
	}
	gtk_clist_thaw (GTK_CLIST (filelst));

	if(closedir(dir)==-1) perror("closedir");
	if(current!=NULL) {
		chdir(current);
		free(current);
	}
}


static void sorted_alloc_file_entry(fich_entry **ent, char *name, char *link, 
				    struct stat info, int isdir, 
				    GdkPixmap *pix, GdkBitmap *msk, int icotyp)
{
	char *nom;
	char *u1, *u2;
	char *g1, *g2;
	fich_entry *last, *new, *p;

	nom=(char*)malloc((strlen(name)+strlen(link)+5)*sizeof(char));
	strcpy(nom, name);
	if(strcmp(link, "")!=0) {
		strcat(nom, " -> ");
		strcat(nom, link);
	}
	
	if(*ent==NULL) {
		if(isdir && !opt.sh_hdir && name[0]=='.') return;
		if(!isdir && !opt.sh_hfil && name[0]=='.') return;
		(*ent)=(fich_entry *)malloc(sizeof(fich_entry));
		(*ent)->name=strdup(name);
		(*ent)->txt=nom;
		(*ent)->isdir=isdir;
		(*ent)->info=info;
		(*ent)->ico=icotyp;
		(*ent)->pix=pix;
		(*ent)->msk=msk;
		(*ent)->suiv=NULL;
		}
	else {
		p=*ent;
		switch(opt.tri) {
		   case AFF_NOM:
			if(isdir) {
				if( !opt.sh_hdir && name[0]=='.')
					return;
				while(p!=NULL && p->isdir) {
					if(SUP_OU_INF(opt.tri_inv, strcmp(nom, p->name), 0))
						break;
					last=p;
					p=p->suiv;
				}
			}
			else {
				if( !opt.sh_hfil && name[0]=='.')
					return;
				while(p!=NULL && p->isdir) {
					last=p;
					p=p->suiv;
				}
				while(p!=NULL) {
					if(SUP_OU_INF(opt.tri_inv, strcmp(nom, p->name), 0))
						break;
					last=p;
					p=p->suiv;
				}
			}
			break;
		   case AFF_SIZE:
			while(p!=NULL) {
				if(SUP_OU_INF(opt.tri_inv, info.st_size, p->info.st_size))
					break;
				last=p;
				p=p->suiv;
			}
			/* la deuxime cl de tri est le nom (ordre croissant) */
			while(p!=NULL && p->info.st_size==info.st_size) {
					if(SUP_OU_INF(0, strcmp(nom, p->name), 0))
						break;
					last=p;
					p=p->suiv;
			}
		   	break;
		   case AFF_INOD:
		   	while(p!=NULL) {
				if(SUP_OU_INF(opt.tri_inv, p->info.st_ino, info.st_ino))
					break;
				last=p;
				p=p->suiv;
			}
			break;
		   case AFF_UID:
			u1=getusrname(info.st_uid);
			while(p!=NULL) {
				u2=getusrname(p->info.st_uid);
				if(SUP_OU_INF(opt.tri_inv, strcmp(u1, u2), 0))
					break;
				last=p;
				p=p->suiv;
			}
			/* la deuxime cl de tri est le nom (ordre croissant) */
			while(p!=NULL && p->info.st_uid==info.st_uid) {
					if(SUP_OU_INF(0, strcmp(nom, p->name), 0))
						break;
					last=p;
					p=p->suiv;
			}
		   	break;
		   case AFF_GID:
			g1=getgrpname(info.st_gid);
			while(p!=NULL) {
				g2=getgrpname(p->info.st_gid);
				if(SUP_OU_INF(opt.tri_inv, strcmp(g1, g2), 0))
					break;
				last=p;
				p=p->suiv;
			}
			/* la deuxime cl de tri est le nom utilisateur (ordre croissant) */
			u1=getusrname(info.st_uid);
			while(p!=NULL && p->info.st_gid==info.st_gid) {
				u2=getusrname(p->info.st_uid);
				if(SUP_OU_INF(0, strcmp(u1, u2), 0))
					break;
				last=p;
				p=p->suiv;
			}
			/* la troisime cl de tri est le nom (ordre croissant) */
			while(p!=NULL && p->info.st_gid==info.st_gid && p->info.st_uid==info.st_uid) {
					if(SUP_OU_INF(0, strcmp(nom, p->name), 0))
						break;
					last=p;
					p=p->suiv;
			}
		   	break;
		   default: /* tri par nom (toujours croissant) */
			if( !opt.sh_hfil && name[0]=='.')
				return;
			while(p!=NULL) {
				if(SUP_OU_INF(0, strcmp(nom, p->name), 0))
					break;
				last=p;
				p=p->suiv;
			}
		   	break;
		}
		new=(fich_entry *)malloc(sizeof(fich_entry));
		new->suiv=p;
		new->name=strdup(name);
		new->txt=nom;
		new->isdir=isdir;
		new->info=info;
		new->ico=icotyp;
		new->pix=pix;
		new->msk=msk;
		if(p==*ent) *ent=new;
		else	    last->suiv=new;
	}
}


static char *getusrname(uid_t uid) {
	struct passwd *pwuid;
	struct user *new, *p;
	
	p=userlst;
	while(p!=NULL) {
		if(uid==p->uid) return p->name;
		p=p->next;
	}
	
	new=(struct user *)malloc(sizeof(struct user));
	pwuid=getpwuid(uid);
	new->uid=uid;
	if(pwuid!=NULL) new->name=strdup(pwuid->pw_name);
	else {
		char str[15];
		sprintf(str, "%d", uid);
		new->name=strdup(str);
	}
	new->next=userlst;
	userlst=new;
	return userlst->name;
}


static char *getgrpname(gid_t gid) {
	struct group *grpid;
	struct grpnid *new, *p;
	
	p=grouplst;
	while(p!=NULL) {
		if(gid==p->gid) return p->name;
		p=p->next;
	}
	
	new=(struct grpnid *)malloc(sizeof(struct grpnid));
	grpid=getgrgid(gid);
	new->gid=gid;
	if(grpid!=NULL) new->name=strdup(grpid->gr_name);
	else {
		char str[15];
		sprintf(str, "%d", gid);
		new->name=strdup(str);
	}
	
	new->next=grouplst;
	grouplst=new;
	return grouplst->name;
}
