/* Copyright (C) Emmanuel Varagnat 1998

   Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
   le modifier conformment aux dispositions de la Licence Publique Gnrale
   GNU, telle que publie par la Free Software Foundation ; version 2 de la
   licence, ou encore ( votre choix) toute version ultrieure.

   Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE
   GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou
   D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence
   Publique Gnrale GNU .

   Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en
   mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
   Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.

  ==========================================================================

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ==========================================================================

   Ecrit par Emmanuel Varagnat (emmanuel.varagnat@altavista.net)  */

#include <gtk/gtk.h>
#include <unistd.h>
#include <stdio.h>
#include <mntent.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <errno.h>

#include "arbo.h"
#include "dialogs.h"
#include "explore.h"
#include "files.h"
#include "log.h"

#include "../images/drive.xpm"
#include "../images/floppy.xpm"
#include "../images/vfat.xpm"
#include "../images/iso9660.xpm"
#include "../images/hdd.xpm"
#include "../images/proc.xpm"
#include "../images/nfs.xpm"
#include "../images/scsi.xpm"
#include "../images/mounted.xpm"
#include "../images/not_mounted.xpm"

static void load_mount_icon();
static GtkWidget *mount_item_menu(gchar*, GdkPixmap*, GdkBitmap*, int);
static void update_mount_menu(GtkMenuItem*, GtkWidget*);
static void mount_menu_handler(GtkWidget*, gchar*);
static int is_mounted(char*);
static int is_in_subdir(char*, char*);

static GdkPixmap *pix_floppy, *pix_hdd, *pix_iso9660, *pix_vfat, *pix_proc, *pix_nfs, *pix_scsi;
static GdkBitmap *msk_floppy, *msk_hdd, *msk_iso9660, *msk_vfat, *msk_proc, *msk_nfs, *msk_scsi;
static GdkPixmap *pix_pucep, *pix_pucev;
static GdkBitmap *msk_puce;

GtkWidget *create_mount_menu() {
	GtkWidget *mount_bar, *mount_menu, *root_item, *item;
	GtkWidget *arrow, *arrow_hbox, *pix;
	GtkWidget *mntitem;
	GdkPixmap *xpm;
	GdkBitmap *msk;
	FILE *fp;
	struct mntent *mntent;
	char txt[100];
	int n=0;

	load_mount_icon();
	
	/* cree le menu */
	mount_menu=gtk_menu_new();

	/* recupere les infos de getmntent */
	if((fp=setmntent("/etc/fstab", "r"))) {
	   while((mntent=getmntent(fp))) {
	       char tmp[10]="";

	       sprintf(txt, "%s   %s", mntent->mnt_fsname, mntent->mnt_dir);
	       item=gtk_menu_item_new();
	       if(strcmp(mntent->mnt_dir, "/proc")==0)
	    	   mntitem=mount_item_menu(txt, pix_proc, msk_proc, 0);
	       else {
		   if(strlen(mntent->mnt_fsname)>=5)
			   strcpy(tmp, mntent->mnt_fsname+5);
		   if(tmp[0]=='f' && tmp[1]=='d')
		       mntitem=mount_item_menu(txt, pix_floppy, msk_floppy, 0);
		   else {
		       if(tmp[0]=='s' && ((tmp[1]=='c' && (tmp[2]=='d' || tmp[2]=='h')) || tmp[1]=='d' || tmp[1]=='t' || tmp[1]=='r'))
			   mntitem=mount_item_menu(txt, pix_scsi, msk_scsi, 0);
		       else {		    
			   if(strcmp(mntent->mnt_type, "iso9660")==0)
			       mntitem=mount_item_menu(txt, pix_iso9660, msk_iso9660, 0);
			   else {
			       if(strcmp(mntent->mnt_type, "nfs")==0)
				   mntitem=mount_item_menu(txt, pix_nfs, msk_nfs, 0);
			       else {
				   if(strcmp(mntent->mnt_type, "vfat")==0)
				       mntitem=mount_item_menu(txt, pix_vfat, msk_vfat, 0);
				   else
				       mntitem=mount_item_menu(txt, pix_hdd, msk_hdd, 0);
			       }
			   }
		       }
		   }
	       }
	       gtk_widget_show(mntitem);
	       gtk_container_add(GTK_CONTAINER(item), mntitem);
	       gtk_object_set_user_data(GTK_OBJECT(item), (gpointer)strdup(mntent->mnt_dir));
	       gtk_signal_connect(GTK_OBJECT(item), "activate",
		       GTK_SIGNAL_FUNC(mount_menu_handler), (gpointer)strdup(mntent->mnt_dir));
	       gtk_menu_append(GTK_MENU(mount_menu), item);
	       gtk_widget_show(item);
	       if(!hasmntopt(mntent, "user"))
	    	    gtk_widget_set_sensitive(item, FALSE);
	       n++;
	   }
	   endmntent(fp);
	}
	else {
	   log_text_add(ERREUR, "Ne peut ouvrir le fichier '/etc/fstab'");
	   msg_dialog("Erreur", "Ne peut ouvrir le fichier '/etc/fstab'");
	}
	mount_bar = gtk_menu_bar_new ();
	root_item = gtk_menu_item_new ();
	gtk_signal_connect(GTK_OBJECT(root_item), "activate",
		GTK_SIGNAL_FUNC(update_mount_menu), mount_menu);
	gtk_container_add (GTK_CONTAINER (mount_bar), root_item);
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (root_item), mount_menu);
	arrow_hbox = gtk_hbox_new (FALSE, 1);
	gtk_container_add (GTK_CONTAINER (root_item), arrow_hbox);
	xpm = gdk_pixmap_create_from_xpm_d(mainwin->window, &msk,
						&mainwin->style->bg[GTK_STATE_NORMAL],
						drive_xpm);
	pix=gtk_pixmap_new(xpm, msk);
	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
	gtk_box_pack_start (GTK_BOX (arrow_hbox), arrow, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (arrow_hbox), pix, FALSE, FALSE, 4);
	gtk_misc_set_alignment (GTK_MISC (pix), 0.5, 0.5);
	gtk_misc_set_alignment (GTK_MISC (arrow), 0.5, 0.5);
	gtk_widget_show(pix);
	gtk_widget_show(arrow);
	gtk_widget_show(arrow_hbox);
	gtk_widget_show(root_item);
	gtk_widget_show(mount_bar);

	/* en cas d'erreur ou de fichier ne permettant aucune action  l'utilisateur */
	if(n==0)
	    gtk_widget_set_sensitive(mount_bar, FALSE);

	return mount_bar;
}



static void load_mount_icon() {
	pix_floppy = gdk_pixmap_create_from_xpm_d(mainwin->window, &msk_floppy,
						&mainwin->style->bg[GTK_STATE_NORMAL],
						floppy_xpm);
	pix_iso9660 = gdk_pixmap_create_from_xpm_d(mainwin->window, &msk_iso9660,
						&mainwin->style->bg[GTK_STATE_NORMAL],
						iso9660_xpm);
	pix_hdd = gdk_pixmap_create_from_xpm_d(mainwin->window, &msk_hdd,
						&mainwin->style->bg[GTK_STATE_NORMAL],
						hdd_xpm);
	pix_vfat = gdk_pixmap_create_from_xpm_d(mainwin->window, &msk_vfat,
						&mainwin->style->bg[GTK_STATE_NORMAL],
						vfat_xpm);
	pix_proc = gdk_pixmap_create_from_xpm_d(mainwin->window, &msk_proc,
						&mainwin->style->bg[GTK_STATE_NORMAL],
						proc_xpm);
	pix_nfs = gdk_pixmap_create_from_xpm_d(mainwin->window, &msk_nfs,
						&mainwin->style->bg[GTK_STATE_NORMAL],
						nfs_xpm);
	pix_scsi = gdk_pixmap_create_from_xpm_d(mainwin->window, &msk_scsi,
						&mainwin->style->bg[GTK_STATE_NORMAL],
						scsi_xpm);

	pix_pucep = gdk_pixmap_create_from_xpm_d(mainwin->window, &msk_puce,
						&mainwin->style->bg[GTK_STATE_NORMAL],
						mounted_xpm);
	pix_pucev = gdk_pixmap_create_from_xpm_d(mainwin->window, NULL,
						&mainwin->style->bg[GTK_STATE_NORMAL],
						not_mounted_xpm);
}

static GtkWidget *mount_item_menu(gchar *label_text, GdkPixmap *pixmap, GdkBitmap *mask, int state) {
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *pixmapwid, *puce;

	hbox = gtk_hbox_new (FALSE, 0);

	puce = gtk_pixmap_new((state)?pix_pucep:pix_pucev, msk_puce);
	pixmapwid = gtk_pixmap_new (pixmap, mask);
	label = gtk_label_new (label_text);

	gtk_box_pack_start (GTK_BOX (hbox), puce, FALSE, FALSE, 4);
	gtk_box_pack_start (GTK_BOX (hbox), pixmapwid, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 6);

	gtk_widget_show(puce);
	gtk_widget_show(pixmapwid);
	gtk_widget_show(label);
	return (hbox);
}

static void update_mount_menu(GtkMenuItem *widget, GtkWidget *mnt_menu) {
	GList *lst_item, *child;
	GList *box_child;
	char *dta;
	int m;
	
	lst_item=GTK_MENU_SHELL(widget->submenu)->children;
	while(lst_item) {
	       dta=(char*)gtk_object_get_user_data(GTK_OBJECT(lst_item->data));
	       if((m=is_mounted(dta))==-1) {
		  log_text_add(ERREUR, "Ne peut ouvrir '/proc/mounts' ou '/etc/mtab'");
		  msg_dialog("Erreur", "Ne peut ouvrir '/proc/mounts' ou '/etc/mtab'");
		  gtk_widget_set_sensitive(mnt_menu, FALSE);
		  break;
	       }
	       else {	
		  child=gtk_container_children(GTK_CONTAINER(lst_item->data));
		  box_child=GTK_BOX(child->data)->children;
		  gtk_pixmap_set(GTK_PIXMAP(((GtkBoxChild*)(box_child->data))->widget), 
		  			(m)?pix_pucep:pix_pucev, msk_puce);
		  gtk_object_set_data(GTK_OBJECT(lst_item->data), "mounted", (gpointer)((m)?1:0));
	       }
	       lst_item=g_list_next(lst_item);
	}
}

static void mount_menu_handler(GtkWidget *widget, gchar *path) {
     GtkWidget *mntbox;
     GtkWidget *label;
     int mnt, pid, out;
/*     pid_t pid;*/
     char *argv[3];
     
     mnt=(int)gtk_object_get_data(GTK_OBJECT(widget), "mounted");
     switch(pid=fork()) {
	 case -1:
	      msg_dialog("Erreur", strerror(errno));
	      break;
         case 0:
	      out=open("/dev/null", O_RDONLY);
	      close(2);
	      dup( (pipe2[1])?pipe2[1]:out );
	      close(1);
	      dup( (pipe1[1])?pipe1[1]:out );
	      close(out);
	      argv[0]=strdup((mnt)?"umount":"mount");
	      argv[1]=path;
	      argv[2]=NULL;
	      if(execvp((mnt)?"umount":"mount", argv)==-1)
	      	  msg_dialog("Erreur", strerror(errno));
	      break;
	 default:	
	      mntbox=gtk_window_new(GTK_WINDOW_DIALOG);
	      gtk_window_position(GTK_WINDOW(mntbox), GTK_WIN_POS_CENTER);
	      gtk_window_set_title(GTK_WINDOW(mntbox), "Montage/Dmontage");
	      gtk_container_border_width(GTK_CONTAINER(mntbox), 10);
	      gtk_grab_add(mntbox);

	      label=gtk_label_new("Montage/Dmontage en cours ...");
	      gtk_container_add(GTK_CONTAINER(mntbox), label);

	      gtk_widget_show_now(label);
	      gtk_widget_show_now(mntbox);
	      while (gtk_events_pending())
		  gtk_main_iteration();

/*	      gtk_signal_emit_by_name(GTK_OBJECT(mntbox), "expose_event");*/

	      waitpid(pid, NULL, 0);

	      gtk_widget_destroy(mntbox);
	      gtk_object_set_data(GTK_OBJECT(widget), "mounted", (gpointer)((mnt)?0:1));

	      arbo_refresh_dir(path);
	      if(strcmp(select_path, path)==0)
	      	  Refresh_file_list(path);
	      else
	          if(is_in_subdir(path, select_path)==1)
		      arbo_change_dir(path);

	      break;
     }
}

static int is_mounted(char *ent) {
     struct mntent *cur;
     FILE *fp;
     
     if((fp=setmntent("/proc/mounts", "r"))==NULL)
	if((fp=setmntent("/etc/mtab", "r"))==NULL)
	   return -1;
     while((cur=getmntent(fp))) {
     	if(strcmp(cur->mnt_dir, ent)==0) {
	    endmntent(fp);
	    return 1;
	}
     }
     endmntent(fp);
     return 0;
}

/* is est il prefixe de in 	*/
/*             is = in  --> 0	*/
/* is est prefixe de in --> 1	*/
/* 		  sinon --> -1  */
static int is_in_subdir(char *is, char *in) {
     int i=0;
     
     while(is[i]!='\0' && in[i]!='\0' && is[i]==in[i])
        i++;

     if(is[i]=='\0' && in[i]=='\0')
         return 0;
     if(is[i]=='\0')
         return 1;
     else
         return -1;
}
