/* mv -- move or rename files
   Copyright (C) 86, 89, 90, 91, 95, 1996 Free Software Foundation, Inc.

  ==========================================================================

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ==========================================================================

   Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
   le modifier conformment aux dispositions de la Licence Publique Gnrale
   GNU, telle que publie par la Free Software Foundation ; version 2 de la
   licence, ou encore ( votre choix) toute version ultrieure.

   Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE
   GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou
   D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence
   Publique Gnrale GNU .

   Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en
   mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
   Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.

  ==========================================================================

   Written by Mike Parker and David MacKenzie
   From GNU file utils 3.16.

   Modified and cleaned for my use (modifi et nettoy pour mon usage).
   Copyright (C) 1998 - Emmanuel Varagnat (emmanuel.varagnat@altavista.net) */
   
#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <utime.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>

#include "../gnulib/path-concat.h"

#include "types.h"
#include "arbo.h"
#include "dialogs.h"
#include "files.h"

#include "../images/hd1.xpm"
#include "../images/hd2.xpm"

#ifdef HAVE_LCHOWN
# define chown(PATH, OWNER, GROUP) lchown(PATH, OWNER, GROUP)
#endif

char *basename ();
/*enum backup_type get_version ();*/
int isdir ();
int yesno ();
int safe_read ();
int full_write ();
void strip_trailing_slashes ();
int euidaccess ();

static int movefile();

/* If nonzero, query the user before overwriting files. */
static int interactive;

/* If nonzero, do not query the user before overwriting unwritable
   files. */
static int override_mode;

/* If nonzero, do not move a nondirectory that has an existing destination
   with the same or newer modification time. */
static int update = 0;

/* This process's effective user ID.  */
static uid_t myeuid;

/* FIXME */
static struct stat dest_stats, source_stats;


/************************* GTK **************************/
/*static GtkWidget *merrorwin=NULL;*/
static GtkWidget *mv_win;
static GtkWidget *errlst;
static GtkWidget *lb_src, *lb_dest;
static GdkPixmap *xpm_hd1, *xpm_hd2;
static GdkBitmap *hd_msk;

static void close_rmwin(GtkWidget *widg, gpointer p) {
     gtk_grab_remove(mv_win);
     gtk_widget_destroy(mv_win);
}

static void add_error(const char *file, const char *msg) {
     char *row[2];

     if (!GTK_WIDGET_VISIBLE(errlst)) {
	     gtk_widget_show(errlst);
     }
     gtk_clist_freeze(GTK_CLIST(errlst));
     row[0]=file;
     row[1]=msg;
     gtk_clist_append(GTK_CLIST(errlst), row);
     gtk_clist_thaw(GTK_CLIST(errlst));

/*	while (gtk_events_pending())
	    gtk_main_iteration();*/
}

void cmd_move_files(char *dest_dir) {
     GtkWidget *vbox, *hbox, *vbox_lb;
     GtkWidget *button, *pix;
     GtkWidget *sep;
/*     int c;*/
     GList *p;
     gint row;
     char src[200], *tt[2]={ " ", " "};
     int errors;
     int nb_err=0, err;
/*     int make_backups = 0;*/

     myeuid = geteuid ();
     interactive = override_mode = update = 0;
     errors = 0;

/*     if (argc > optind + 2 && !isdir (argv[argc - 1]))
       error (1, 0,
	      _("when moving multiple files, last argument must be a directory"));*/

     mv_win = gtk_window_new( GTK_WINDOW_DIALOG );
     gtk_window_set_title(GTK_WINDOW(mv_win), "Supprimer");
     gtk_window_position( GTK_WINDOW(mv_win), GTK_WIN_POS_CENTER );
     gtk_widget_set_usize(mv_win, 380, 0);
     gtk_signal_connect (GTK_OBJECT (mv_win), "delete_event",
	                    GTK_SIGNAL_FUNC (close_rmwin), NULL);
     gtk_container_border_width(GTK_CONTAINER(mv_win), 10);
     gtk_widget_realize(mv_win);

     vbox=gtk_vbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER(mv_win), vbox);
     gtk_widget_show(vbox);

     hbox=gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 4);
     gtk_widget_show(hbox);

     xpm_hd1 = gdk_pixmap_create_from_xpm_d(mv_win->window, &hd_msk,
						&mv_win->style->bg[GTK_STATE_NORMAL],
						hd1_xpm);
     xpm_hd2 = gdk_pixmap_create_from_xpm_d(mv_win->window, &hd_msk,
						&mv_win->style->bg[GTK_STATE_NORMAL],
						hd2_xpm);

     pix=gtk_pixmap_new(xpm_hd1, hd_msk);
     gtk_box_pack_start(GTK_BOX(hbox), pix, FALSE, TRUE, 4);
     gtk_widget_show(pix);

     vbox_lb=gtk_vbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox_lb, FALSE, TRUE, 4);
     gtk_widget_show(vbox_lb);

     lb_src=gtk_label_new("");
     gtk_label_set_justify (GTK_LABEL(lb_src), GTK_JUSTIFY_LEFT);
     gtk_box_pack_start(GTK_BOX(vbox_lb), lb_src, FALSE, TRUE, 4);
     gtk_widget_show(lb_src);

     lb_dest=gtk_label_new("");
     gtk_label_set_justify (GTK_LABEL(lb_dest), GTK_JUSTIFY_LEFT);
     gtk_box_pack_start(GTK_BOX(vbox_lb), lb_dest, FALSE, TRUE, 4);
     gtk_widget_show(lb_dest);

     button=gtk_button_new_with_label("Annuler");
     gtk_signal_connect (GTK_OBJECT (button), "clicked",
	                    GTK_SIGNAL_FUNC (close_rmwin), NULL);
     gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 4);
     gtk_widget_show(button);

     sep=gtk_hseparator_new();
     gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, TRUE, 4);
     gtk_widget_hide(sep);

     errlst=gtk_clist_new_with_titles(2, tt);
     gtk_clist_column_titles_show(GTK_CLIST(errlst));
/*     gtk_clist_set_selection_mode(GTK_CLIST(errlst), GTK_SELECTION_EXTENDED);*/
     gtk_clist_set_column_width(GTK_CLIST(errlst), 0, 120);
     gtk_box_pack_start(GTK_BOX(vbox), errlst, TRUE, TRUE, 4);
     gtk_widget_hide(errlst);

     gtk_grab_add(mv_win);

     gtk_widget_realize(vbox);
     gtk_widget_realize(sep);
     gtk_widget_realize(errlst);
     gtk_widget_show_now(mv_win);

     if(chdir(select_path)) {
	 msg_dialog("Erreur", strerror(errno));
	 gtk_grab_remove(mv_win);
	 gtk_widget_destroy(mv_win);
	 return;
     }

     /* Move each arg but the last onto the last. */
     p=GTK_CLIST(filelst)->selection;
     while(p!=NULL) {
	  char *nom;

	  row=(int)p->data;
	  nom=strdup(((File_data*)gtk_clist_get_row_data(GTK_CLIST(filelst), row))->nom);

 	  strip_trailing_slashes (nom);

	  strcpy(src, select_path);
	  strcat(src, "/");
	  strcat(src, nom);
	  gtk_label_set_text(GTK_LABEL(lb_src), src);

	  gtk_label_set_text(GTK_LABEL(lb_dest), dest_dir);
	/*printf("%s -> %s\n", txt, dest_dir);*/
	  err = movefile (src, dest_dir);
	  if(err==2)
	     break;
	  nb_err |= err;
	  p=p->next;
     }

     if(!nb_err) {
	  gtk_grab_remove(mv_win);
	  gtk_widget_destroy(mv_win);
     }
}


/* If PATH is an existing directory, return nonzero, else 0.  */

static int
is_real_dir (const char *path)
{
  struct stat stats;

  return lstat (path, &stats) == 0 && S_ISDIR (stats.st_mode);
}


/* Copy regular file SOURCE onto file DEST.
   Return 1 if an error occurred, 0 if successful. */

static int
copy_reg (const char *source, const char *dest)
{
  int ifd;
  int ofd;
  char buf[1024 * 8];
  int len;			/* Number of bytes read into `buf'. */

  if (!S_ISREG (source_stats.st_mode))
    {
      add_error(source, "fichier non rgulier: ne peut dplacer le fichier d'un systme de fichier  un autre");
      return 1;
    }

  if (unlink (dest) && errno != ENOENT)
    {
      add_error(dest, "ne peut effacer le fichier");
      return 1;
    }

  ifd = open (source, O_RDONLY, 0);
  if (ifd < 0)
    {
      add_error(source, strerror(errno));
      return 1;
    }
  ofd = open (dest, O_WRONLY | O_CREAT | O_TRUNC, 0600);
  if (ofd < 0)
    {
      add_error(dest, strerror(errno));
      close (ifd);
      return 1;
    }

  while ((len = safe_read (ifd, buf, sizeof (buf))) > 0)
    {
      if (full_write (ofd, buf, len) < 0)
	{
	  add_error(dest, strerror(errno));
	  close (ifd);
	  close (ofd);
	  unlink (dest);
	  return 1;
	}
    }
  if (len < 0)
    {
      add_error(source, strerror(errno));
      close (ifd);
      close (ofd);
      unlink (dest);
      return 1;
    }

  if (close (ifd) < 0)
    {
      add_error(source, strerror(errno));
      close (ofd);
      return 1;
    }
  if (close (ofd) < 0)
    {
      add_error(dest, strerror(errno));
      return 1;
    }

  /* chown turns off set[ug]id bits for non-root,
     so do the chmod last.  */

  /* Try to copy the old file's modtime and access time.  */
  {
    struct utimbuf tv;

    tv.actime = source_stats.st_atime;
    tv.modtime = source_stats.st_mtime;
    if (utime (dest, &tv))
      {
        add_error(dest, strerror(errno));
	return 1;
      }
  }

  /* Try to preserve ownership.  For non-root it might fail, but that's ok.
     But root probably wants to know, e.g. if NFS disallows it.  */
  if (chown (dest, source_stats.st_uid, source_stats.st_gid)
      && (errno != EPERM || myeuid == 0))
    {
      add_error(dest, strerror(errno));
      return 1;
    }

  if (chmod (dest, source_stats.st_mode & 07777))
    {
      add_error(dest, strerror(errno));
      return 1;
    }

  return 0;
}


/* Move SOURCE onto DEST.  Handles cross-filesystem moves.
   If DEST is a directory, SOURCE must be also.
   Return 0 if successful, 1 if an error occurred.  */

static int
do_move (const char *source, const char *dest)
{
/*  char *dest_backup = NULL;*/

  if (lstat (source, &source_stats) != 0)
    {
      add_error(source, strerror(errno));
      return 1;
    }

  if (lstat (dest, &dest_stats) == 0)
    {
      if (source_stats.st_dev == dest_stats.st_dev
	  && source_stats.st_ino == dest_stats.st_ino)
	{
	  add_error(source, "fichiers source et destination identiques");
	  return 1;
	}

      if (S_ISDIR (dest_stats.st_mode))
	{
	  char txt[100];
	  sprintf("ne peut craser le rpertoire '%s'", dest);
	  add_error(dest, txt);
	  return 1;
	}

      if (!S_ISDIR (source_stats.st_mode) && update
	  && source_stats.st_mtime <= dest_stats.st_mtime)
	return 0;

      if (!override_mode && interactive
	  && euidaccess (dest, W_OK))
	{
	  char txt[100], *lab[]={ "Oui", "Non", "Tout", "Annuler", NULL };
	  sprintf(txt, "Overriding mode %04o for\n%s?",
	  			(unsigned int) (dest_stats.st_mode & 07777),
				dest);
	  switch(buttons_dialog(txt, "", lab)) {
	     /*Non*/   case 1: return 0;
	     /*Tout*/  case 2:
	   		  override_mode=1;
	   		  break;
	     /*Annuler*/case 3:
	   		  return 2;
	   		  break;
	  }
	}
      else if (interactive)
	{
	  char txt[100], *lab[]={ "Oui", "Non", "Tout", "Annuler", NULL };
	  sprintf(txt, "Remplacer '%s' ?", dest);
	  switch(buttons_dialog(txt, "", lab)) {
	     /*Non*/   case 1: return 0;
	     /*Tout*/  case 2:
	   		  interactive=1;
	   		  break;
	     /*Annuler*/case 3:
	   		  return 2;
	   		  break;
	  }
	}
    }
  else if (errno != ENOENT)
    {
      add_error(dest, strerror(errno));
      return 1;
    }

  if (rename (source, dest) == 0)
    {
      return 0;
    }

  if (errno != EXDEV)
    {
      add_error(source, "Ne peut dplacer le fichier");
/*      error (0, errno, _("cannot move `%s' to `%s'"), source, dest);*/
      return 1;
    }

  /* rename failed on cross-filesystem link.  Copy the file instead. */

  if (copy_reg (source, dest))
    return 1;

  if (unlink (source))
    {
      add_error(source, "Ne peut effacer le fichier");
      return 1;
    }

  return 0;
}

/* Move file SOURCE onto DEST.  Handles the case when DEST is a directory.
   Return 0 if successful, 1 if an error occurred.  
   And 2 if cancel */

static int
movefile (const char *source, const char *dest)
{
  strip_trailing_slashes (source);

  if ((dest[strlen (dest) - 1] == '/' && !is_real_dir (source))
      || isdir (dest))
    {
      /* Target is a directory; build full target filename. */
      char *base;
      char *new_dest;
      int fail;

      base = basename (source);
      new_dest = path_concat (dest, base, NULL);
      if (new_dest == NULL)
	msg_dialog("Erreur", "Plus de mmoire disponible");
      fail = do_move (source, new_dest);
      free (new_dest);
      return fail;
    }
  else
    return do_move (source, dest);
}
