/* Copyright (C) Emmanuel Varagnat 1998

   Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
   le modifier conformment aux dispositions de la Licence Publique Gnrale
   GNU, telle que publie par la Free Software Foundation ; version 2 de la
   licence, ou encore ( votre choix) toute version ultrieure.

   Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE
   GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou
   D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence
   Publique Gnrale GNU .

   Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en
   mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
   Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.

  ==========================================================================

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  ==========================================================================

   Ecrit par Emmanuel Varagnat (emmanuel.varagnat@altavista.net)  */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <unistd.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <sys/stat.h>
#include <dirent.h>
#include <errno.h>
#include <error.h>
#include <time.h>

#include "types.h"
#include "arbo.h"
#include "dialogs.h"
#include "explore.h"
#include "files.h"

static void size_subdir(char*);
static int recursive_chmod(char*);

static GtkWidget *propwin, *l_total;
static struct stat info, sav;
static long total, nb_file, nb_dir;
static int op, recurs;

static void close_info(GtkWidget *w, gpointer p) {
	gtk_grab_remove(propwin);
	gtk_widget_destroy(propwin);
}

static void rien(GtkWidget *w, gpointer p) {};

static void file_change_info(GtkWidget *w, gchar *nom) {
	int nblnk=0;

	if(g_list_length (GTK_CLIST(filelst)->selection)>1) {
		if(info.st_mode!=0) {
			GList *p=GTK_CLIST(filelst)->selection;
			struct stat stfil;
			char path[255], *fname;
			int row;
			
			while(p!=NULL) {
				row=(int)p->data;
				fname=((File_data*)gtk_clist_get_row_data(GTK_CLIST(filelst), row))->nom;
				strcpy(path, select_path);
				strcat(path, "/");
				strcat(path, fname);

				lstat(path, &stfil);
				if(S_ISLNK(stfil.st_mode))
					nblnk++;
				else {
					switch(op) {
					    case 0:
						if(chmod(path, info.st_mode|stfil.st_mode)==-1)
							multierror_dialog(fname, strerror(errno));
						break;
					    case 1:
						if(chmod(path, stfil.st_mode&(~info.st_mode))==-1)
							multierror_dialog(fname, strerror(errno));
						break;
					    case 2:
						if(chmod(path, info.st_mode)==-1)
							multierror_dialog(fname, strerror(errno));
						break;
					}
					if(recurs && S_ISDIR(stfil.st_mode)) {
						nblnk+=recursive_chmod(path);
					}
				}
				
				p=p->next;
			}
			Refresh_file_list(select_path);
			if(nblnk) {
				char slnk[30];
				sprintf(slnk, "%d liens ont t rencontr(s).", nblnk);
				msg_dialog("Informations", slnk);
			}
		}
	}
	else {
		if(sav.st_mode!=info.st_mode) {
			char path[255];	

			strcpy(path, select_path);
			strcat(path, "/");
			strcat(path, nom);
			if(chmod(path, info.st_mode)==-1) perror("chmod");
			Refresh_file_list(select_path);
		}
	}
	gtk_grab_remove(propwin);
	gtk_widget_destroy(propwin);
}

static void dir_change_info(GtkWidget *w, gchar *nom) {
	if(sav.st_mode!=info.st_mode) {
		char dr[]=" ---------- ";

		if(chmod(nom, info.st_mode)==-1) perror("chmod");
		stat2strmode(info, dr);
		statusbar_display(1, dr);
/*		Refresh_file_list(select_path);*/
	}
	gtk_grab_remove(propwin);
	gtk_widget_destroy(propwin);
}

static gint keyhandler(GtkWidget *widget, GdkEventKey *event, GtkWidget *win) {
	if(event->keyval==GDK_Escape) {
		gtk_grab_remove(win);
		gtk_widget_destroy(win);
		return TRUE;
	}
	return FALSE;
}

static void permission_changed(GtkWidget *widget, long perm) {
	info.st_mode=info.st_mode^perm;
}

static void operation_changed(GtkWidget *widget, int oper) {
	op=oper;
}

static void recursive_state(GtkWidget *widget, gpointer p) {
	recurs=!recurs;
}

static void permission_table(GtkWidget *tbl, int multi) {
	GtkWidget *c_perm, *l_perm;

	l_perm=gtk_label_new("User");
	gtk_table_attach_defaults(GTK_TABLE(tbl), l_perm, 0, 1, 0, 1);
	gtk_widget_show(l_perm);
	l_perm=gtk_label_new("Group");
	gtk_table_attach_defaults(GTK_TABLE(tbl), l_perm, 1, 2, 0, 1);
	gtk_widget_show(l_perm);
	l_perm=gtk_label_new("Other");
	gtk_table_attach_defaults(GTK_TABLE(tbl), l_perm, 2, 3, 0, 1);
	gtk_widget_show(l_perm);
	l_perm=gtk_label_new("Special");
	gtk_table_attach_defaults(GTK_TABLE(tbl), l_perm, 3, 4, 0, 1);
	gtk_widget_show(l_perm);
		
	c_perm=gtk_check_button_new_with_label("read");
	if( !multi && (info.st_mode & S_IRUSR)) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (c_perm), TRUE);
	gtk_signal_connect (GTK_OBJECT (c_perm), "clicked",
                               GTK_SIGNAL_FUNC (permission_changed), (gpointer)S_IRUSR);
	gtk_table_attach_defaults(GTK_TABLE(tbl), c_perm, 0, 1, 1, 2);
	gtk_widget_show(c_perm);
	c_perm=gtk_check_button_new_with_label("read");
	if( !multi && (info.st_mode & S_IRGRP)) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (c_perm), TRUE);
	gtk_signal_connect (GTK_OBJECT (c_perm), "clicked",
                               GTK_SIGNAL_FUNC (permission_changed), (gpointer)S_IRGRP);
	gtk_table_attach_defaults(GTK_TABLE(tbl), c_perm, 1, 2, 1, 2);
	gtk_widget_show(c_perm);
	c_perm=gtk_check_button_new_with_label("read");
	if( !multi && (info.st_mode & S_IROTH)) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (c_perm), TRUE);
	gtk_signal_connect (GTK_OBJECT (c_perm), "clicked",
                               GTK_SIGNAL_FUNC (permission_changed), (gpointer)S_IROTH);
	gtk_table_attach_defaults(GTK_TABLE(tbl), c_perm, 2, 3, 1, 2);
	gtk_widget_show(c_perm);
	c_perm=gtk_check_button_new_with_label("Set UID");
	if( !multi && (info.st_mode & S_ISUID)) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (c_perm), TRUE);
	gtk_signal_connect (GTK_OBJECT (c_perm), "clicked",
                               GTK_SIGNAL_FUNC (permission_changed), (gpointer)S_ISUID);
	gtk_table_attach_defaults(GTK_TABLE(tbl), c_perm, 3, 4, 1, 2);
	gtk_widget_show(c_perm);

	c_perm=gtk_check_button_new_with_label("write");
	if( !multi && (info.st_mode & S_IWUSR)) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (c_perm), TRUE);
	gtk_signal_connect (GTK_OBJECT (c_perm), "clicked",
                               GTK_SIGNAL_FUNC (permission_changed), (gpointer)S_IWUSR);
	gtk_table_attach_defaults(GTK_TABLE(tbl), c_perm, 0, 1, 2, 3);
	gtk_widget_show(c_perm);
	c_perm=gtk_check_button_new_with_label("write");
	if( !multi && (info.st_mode & S_IWGRP)) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (c_perm), TRUE);
	gtk_signal_connect (GTK_OBJECT (c_perm), "clicked",
                               GTK_SIGNAL_FUNC (permission_changed), (gpointer)S_IWGRP);
	gtk_table_attach_defaults(GTK_TABLE(tbl), c_perm, 1, 2, 2, 3);
	gtk_widget_show(c_perm);
	c_perm=gtk_check_button_new_with_label("write");
	if( !multi && (info.st_mode & S_IWOTH)) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (c_perm), TRUE);
	gtk_signal_connect (GTK_OBJECT (c_perm), "clicked",
                       GTK_SIGNAL_FUNC (permission_changed), (gpointer)S_IWOTH);
	gtk_table_attach_defaults(GTK_TABLE(tbl), c_perm, 2, 3, 2, 3);
	gtk_widget_show(c_perm);
	c_perm=gtk_check_button_new_with_label("Set GID");
	if( !multi && (info.st_mode & S_ISGID)) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (c_perm), TRUE);
	gtk_signal_connect (GTK_OBJECT (c_perm), "clicked",
                       GTK_SIGNAL_FUNC (permission_changed), (gpointer)S_ISGID);
	gtk_table_attach_defaults(GTK_TABLE(tbl), c_perm, 3, 4, 2, 3);
	gtk_widget_show(c_perm);

	c_perm=gtk_check_button_new_with_label("exec");
	if( !multi && (info.st_mode & S_IXUSR)) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (c_perm), TRUE);
	gtk_signal_connect (GTK_OBJECT (c_perm), "clicked",
                       GTK_SIGNAL_FUNC (permission_changed), (gpointer)S_IXUSR);
	gtk_table_attach_defaults(GTK_TABLE(tbl), c_perm, 0, 1, 3, 4);
	gtk_widget_show(c_perm);
	c_perm=gtk_check_button_new_with_label("exec");
	if( !multi && (info.st_mode & S_IXGRP)) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (c_perm), TRUE);
	gtk_signal_connect (GTK_OBJECT (c_perm), "clicked",
                       GTK_SIGNAL_FUNC (permission_changed), (gpointer)S_IXGRP);
	gtk_table_attach_defaults(GTK_TABLE(tbl), c_perm, 1, 2, 3, 4);
	gtk_widget_show(c_perm);
	c_perm=gtk_check_button_new_with_label("exec");
	if( !multi && (info.st_mode & S_IXOTH)) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (c_perm), TRUE);
	gtk_signal_connect (GTK_OBJECT (c_perm), "clicked",
                       GTK_SIGNAL_FUNC (permission_changed), (gpointer)S_IXOTH);
	gtk_table_attach_defaults(GTK_TABLE(tbl), c_perm, 2, 3, 3, 4);
	gtk_widget_show(c_perm);
	c_perm=gtk_check_button_new_with_label("Sticky");
	if( !multi && (info.st_mode & S_ISVTX)) gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (c_perm), TRUE);
	gtk_signal_connect (GTK_OBJECT (c_perm), "clicked",
                       GTK_SIGNAL_FUNC (permission_changed), (gpointer)S_ISVTX);
	gtk_table_attach_defaults(GTK_TABLE(tbl), c_perm, 3, 4, 3, 4);
	gtk_widget_show(c_perm);
}

static void button_box(GtkWidget *box, char *nom, void (*OK_func)()) {
	GtkWidget *bt;
	GtkWidget *hbox, *hsep;

	hsep=gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(box), hsep, FALSE, TRUE, 5);
	gtk_widget_show(hsep);

	hbox=gtk_hbox_new(TRUE, 10);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, TRUE, 5);
	gtk_widget_show(hbox);

	bt=gtk_button_new_with_label("Ok");
	gtk_signal_connect (GTK_OBJECT (bt), "clicked",
                               GTK_SIGNAL_FUNC (OK_func), nom);
	gtk_box_pack_start(GTK_BOX(hbox), bt, FALSE, TRUE, 0);
	gtk_widget_show(bt);

	bt=gtk_button_new_with_label("Annuler");
	gtk_signal_connect (GTK_OBJECT (bt), "clicked",
                              GTK_SIGNAL_FUNC (close_info), NULL);
	gtk_box_pack_start(GTK_BOX(hbox), bt, FALSE, TRUE, 0);
	gtk_widget_show(bt);

	bt=gtk_button_new_with_label("Infos+");
	gtk_widget_set_sensitive(bt, FALSE);
/*	gtk_signal_connect_object (GTK_OBJECT (bt), "clicked",
                              GTK_SIGNAL_FUNC (ShowFSInfo), (GtkObject*)nom);*/
	gtk_box_pack_start(GTK_BOX(hbox), bt, FALSE, TRUE, 0);
	gtk_widget_show(bt);
}

void ShowFileInfo() {
	GtkWidget *vbox, *hbox;
	GtkWidget *l_path, *l_size, *l_txt;
	GdkCursor *curs;
	int row;
	char *nom, str[255], str_nom[255];

	propwin = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_window_set_title(GTK_WINDOW(propwin), "Proprit");
	gtk_signal_connect (GTK_OBJECT (propwin), "key_press_event",
                               GTK_SIGNAL_FUNC (keyhandler), propwin);
	gtk_signal_connect (GTK_OBJECT (propwin), "delete_event",
                               GTK_SIGNAL_FUNC (rien), NULL);
	gtk_widget_set_sensitive(propwin, FALSE);
	gtk_widget_realize(propwin);
	gtk_grab_add(propwin);
		
	vbox=gtk_vbox_new(FALSE, 0);
	gtk_container_border_width(GTK_CONTAINER(propwin), 10);
	gtk_container_add(GTK_CONTAINER(propwin), vbox);
	gtk_widget_show(vbox);

	if(g_list_length (GTK_CLIST(filelst)->selection)>1) {
		GtkWidget *tbl;
		GtkWidget *c_srperm;
		GtkWidget *tbl_info, *l_txt, *l_seltotal;
		GtkWidget *frame, *framebox;
		GtkWidget *radio1, *radio2, *radio3;
		GList *p;
		long sel_total=0;
		int nb;
		
		op=0;
		
		hbox=gtk_hbox_new(FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
		gtk_widget_show(hbox);
		sprintf(str, "%ld octets", info.st_size);
		l_seltotal=gtk_label_new("");
		gtk_label_set_justify (GTK_LABEL(l_seltotal), GTK_JUSTIFY_LEFT);
		gtk_box_pack_start(GTK_BOX(hbox), l_seltotal, FALSE, TRUE, 0);
		gtk_widget_show(l_seltotal);

		tbl_info=gtk_table_new(1, 3, FALSE);
		gtk_box_pack_start(GTK_BOX(vbox), tbl_info, FALSE, TRUE, 0);
		gtk_widget_show(tbl_info);

		l_total=gtk_label_new("");
		gtk_label_set_justify (GTK_LABEL(l_total), GTK_JUSTIFY_LEFT);
		gtk_table_attach(GTK_TABLE(tbl_info), l_total, 0, 1, 0, 1, GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);
		gtk_widget_show(l_total);
		l_txt=gtk_label_new(" octets au total");
		gtk_label_set_justify (GTK_LABEL(l_txt), GTK_JUSTIFY_LEFT);
		gtk_table_attach(GTK_TABLE(tbl_info), l_txt, 1, 2, 0, 1, GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);
		gtk_widget_show(l_txt);
		l_txt=gtk_label_new("");
		gtk_label_set_justify (GTK_LABEL(l_txt), GTK_JUSTIFY_LEFT);
		gtk_table_attach(GTK_TABLE(tbl_info), l_txt, 2, 3, 0, 1, GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);
		gtk_widget_show(l_txt);

		tbl=gtk_table_new(4, 4, TRUE);
		gtk_box_pack_start(GTK_BOX(vbox), tbl, TRUE, FALSE, 10);
		gtk_widget_show(tbl);

/*		if(access(select_path, W_OK)!=0) {
			gtk_widget_set_sensitive(tbl, FALSE);
		}*/

		permission_table(tbl, 1);

		frame=gtk_frame_new("Oprations ");
		gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 0);
		framebox=gtk_hbox_new(FALSE, 0);
		gtk_container_add(GTK_CONTAINER(frame), framebox);
		gtk_widget_show(framebox);
		radio1=gtk_radio_button_new_with_label(NULL, "ajoute");
		gtk_signal_connect (GTK_OBJECT (radio1), "clicked",
                       GTK_SIGNAL_FUNC (operation_changed), (gpointer)0);
		gtk_box_pack_start(GTK_BOX(framebox), radio1, FALSE, TRUE, 10);
		gtk_widget_show(radio1);
		radio2=gtk_radio_button_new_with_label(
                        gtk_radio_button_group (GTK_RADIO_BUTTON (radio1)),
                        "supprime");
		gtk_signal_connect (GTK_OBJECT (radio2), "clicked",
                       GTK_SIGNAL_FUNC (operation_changed), (gpointer)1);
		gtk_box_pack_start(GTK_BOX(framebox), radio2, FALSE, TRUE, 0);
		gtk_widget_show(radio2);
		radio3=gtk_radio_button_new_with_label(
                        gtk_radio_button_group (GTK_RADIO_BUTTON (radio2)),
                        "galit");
		gtk_signal_connect (GTK_OBJECT (radio3), "clicked",
                       GTK_SIGNAL_FUNC (operation_changed), (gpointer)2);
		gtk_box_pack_start(GTK_BOX(framebox), radio3, FALSE, TRUE, 10);
		gtk_widget_show(radio3);
		gtk_widget_show(frame);
					 	
		recurs=0;
		c_srperm=gtk_check_button_new_with_label("Appliquer les droits dans les sous-rpertoires");
		gtk_signal_connect (GTK_OBJECT (c_srperm), "clicked",
                       GTK_SIGNAL_FUNC (recursive_state), NULL);
/*		gtk_widget_set_sensitive(c_srperm, FALSE);*/
		gtk_box_pack_start(GTK_BOX(vbox), c_srperm, FALSE, TRUE, 10);
		gtk_widget_show(c_srperm);

		button_box(vbox, nom, file_change_info);

		gtk_widget_show(propwin);

		nb=1;
		nb_dir = nb_file = 0;
		total=0;
		p=GTK_CLIST(filelst)->selection;
		while(p!=NULL) {
			row=(int)p->data;
			nom=((File_data*)gtk_clist_get_row_data(GTK_CLIST(filelst), row))->nom;
			strcpy(str_nom, select_path);
			strcat(str_nom, "/");
			strcat(str_nom, nom);
			stat(str_nom, &info);
			sel_total+=info.st_size;
			sprintf(str, "%d fichiers, %ld octets", nb, sel_total);
			gtk_label_set_text(GTK_LABEL(l_seltotal), str);
			while (gtk_events_pending())   /* empeche les erreurs venant */
				gtk_main_iteration();  /* de gtk_label_expose        */
			if(S_ISDIR(info.st_mode)) {
				size_subdir(str_nom);
				nb_dir++;
			} else nb_file++;
			p=p->next;
			nb++;
		}
		sprintf(str, "%ld", total+sel_total);
		gtk_label_set_text(GTK_LABEL(l_total), str);
		sprintf(str, ", %ld fichiers et %ld rpertoires", nb_file, nb_dir);
		gtk_label_set_text(GTK_LABEL(l_txt), str);
		info.st_mode=0;
	}
	else {
		GtkWidget *tbl_info;
		GtkWidget *l_info, *e_nom, *tbl;
		char *ret, str_ctime[30], str_atime[30], str_mtime[30];
		
		l_path=gtk_label_new(select_path);
		gtk_box_pack_start(GTK_BOX(vbox), l_path, TRUE, TRUE, 0);
		gtk_widget_show(l_path);

		/* recupere le nom du fichier */
		row=(int)(GTK_CLIST(filelst)->selection)->data;
		nom=((File_data*)gtk_clist_get_row_data(GTK_CLIST(filelst), row))->nom;
		e_nom=gtk_entry_new();
		gtk_entry_set_text(GTK_ENTRY(e_nom), nom);
		gtk_box_pack_start(GTK_BOX(vbox), e_nom, FALSE, TRUE, 5);
		gtk_widget_show(e_nom);

		strcpy(str_nom, select_path);
		strcat(str_nom, "/");
		strcat(str_nom, nom);
		stat(str_nom, &info);
		sav=info;

		hbox=gtk_hbox_new(FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
		gtk_widget_show(hbox);
		sprintf(str, "%ld octets", info.st_size);
		l_size=gtk_label_new(str);
		gtk_label_set_justify (GTK_LABEL(l_size), GTK_JUSTIFY_LEFT);
		gtk_box_pack_start(GTK_BOX(hbox), l_size, FALSE, TRUE, 0);
		gtk_widget_show(l_size);

		tbl_info=gtk_table_new(1, 2, FALSE);
		gtk_box_pack_start(GTK_BOX(vbox), tbl_info, FALSE, TRUE, 0);
		gtk_widget_show(tbl_info);

		l_total=gtk_label_new("");
		gtk_label_set_justify (GTK_LABEL(l_total), GTK_JUSTIFY_LEFT);
		gtk_table_attach(GTK_TABLE(tbl_info), l_total, 0, 1, 0, 1, GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);
		l_txt=gtk_label_new(" octets dans les sous rpertoires");
		gtk_label_set_justify (GTK_LABEL(l_txt), GTK_JUSTIFY_LEFT);
		gtk_table_attach(GTK_TABLE(tbl_info), l_txt, 1, 2, 0, 1, GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);

		ret=ctime(&info.st_ctime);
		memccpy(str_ctime, ret, '\0', 29);
		ret=ctime(&info.st_atime);
		memccpy(str_atime, ret, '\0', 29);
		ret=ctime(&info.st_mtime);
		memccpy(str_mtime, ret, '\0', 29);
		sprintf(str, "\ndate de changement:   %s"
			     "date d'acces:         %s"
			     "date de modification: %s", str_ctime, str_atime, str_mtime);
		
		l_info=gtk_label_new(str);
		gtk_label_set_justify (GTK_LABEL(l_info), GTK_JUSTIFY_LEFT);
		gtk_box_pack_start(GTK_BOX(vbox), l_info, TRUE, FALSE, 5);
		gtk_widget_show(l_info);
		gtk_widget_map(l_info);

		/* Affiche les permissions */
		tbl=gtk_table_new(4, 4, TRUE);
		gtk_box_pack_start(GTK_BOX(vbox), tbl, TRUE, FALSE, 0);
		gtk_widget_show(tbl);

/*		if(access(select_path, W_OK)!=0) {
			gtk_widget_set_sensitive(e_nom, FALSE);
			gtk_widget_set_sensitive(tbl, FALSE);
		}*/
		permission_table(tbl, 0);

		button_box(vbox, nom, file_change_info);

		gtk_widget_show(propwin);
	
		if(S_ISDIR(info.st_mode)) {
			curs=gdk_cursor_new(GDK_WATCH);
			gdk_window_set_cursor (propwin->window, curs);

			total=0;
			gtk_widget_show(l_total);
			gtk_widget_show(l_txt);
			size_subdir(str_nom);
		}
	}
	gtk_signal_connect (GTK_OBJECT (propwin), "delete_event",
                               GTK_SIGNAL_FUNC (close_info), NULL);
	gtk_widget_set_sensitive(propwin, TRUE);
	curs=gdk_cursor_new(GDK_LEFT_PTR);
	gdk_window_set_cursor (propwin->window, curs);
}

void ShowDirInfo() {
	GtkWidget *vbox, *hbox;
	GtkWidget *l_size, *l_txt;
	GtkWidget *tbl_info;
	GtkWidget *l_info, *e_nom, *tbl;
	GdkBitmap *mask;
	GdkPixmap *pix;
	GdkCursor *curs;
	GtkCTreeNode *tree_sel;
	char *nom, str[255], *str_nom;
	char *ret, str_ctime[30], str_atime[30], str_mtime[30];
	guint8 spc;
		
	propwin = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_window_set_title(GTK_WINDOW(propwin), "Proprit");
	gtk_signal_connect (GTK_OBJECT (propwin), "key_press_event",
                               GTK_SIGNAL_FUNC (keyhandler), propwin);
	gtk_signal_connect (GTK_OBJECT (propwin), "delete_event",
                               GTK_SIGNAL_FUNC (rien), NULL);
	gtk_widget_set_sensitive(propwin, FALSE);
	gtk_widget_realize(propwin);
	gtk_grab_add(propwin);

	vbox=gtk_vbox_new(FALSE, 0);
	gtk_container_border_width(GTK_CONTAINER(propwin), 10);
	gtk_container_add(GTK_CONTAINER(propwin), vbox);
	gtk_widget_show(vbox);

	/* recupere le nom du fichier */
	tree_sel=GTK_CTREE_NODE(GTK_CLIST(arbo)->selection->data);
	gtk_ctree_node_get_pixtext(GTK_CTREE(arbo), tree_sel, 0, &nom, &spc, &pix, &mask);

	e_nom=gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(e_nom), nom);
	gtk_box_pack_start(GTK_BOX(vbox), e_nom, FALSE, TRUE, 5);
	gtk_widget_show(e_nom);

	str_nom=(char *)gtk_ctree_node_get_row_data(GTK_CTREE(arbo), tree_sel);
	stat(str_nom, &info);
	sav=info;

	hbox=gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
	gtk_widget_show(hbox);
	sprintf(str, "%ld octets", info.st_size);
	l_size=gtk_label_new(str);
	gtk_label_set_justify (GTK_LABEL(l_size), GTK_JUSTIFY_LEFT);
	gtk_box_pack_start(GTK_BOX(hbox), l_size, FALSE, TRUE, 0);
	gtk_widget_show(l_size);

	tbl_info=gtk_table_new(1, 2, FALSE);
	gtk_box_pack_start(GTK_BOX(vbox), tbl_info, FALSE, TRUE, 0);
	gtk_widget_show(tbl_info);

	l_total=gtk_label_new("");
	gtk_label_set_justify (GTK_LABEL(l_total), GTK_JUSTIFY_LEFT);
	gtk_table_attach(GTK_TABLE(tbl_info), l_total, 0, 1, 0, 1, GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);
	l_txt=gtk_label_new(" octets dans les sous rpertoires");
	gtk_label_set_justify (GTK_LABEL(l_txt), GTK_JUSTIFY_LEFT);
	gtk_table_attach(GTK_TABLE(tbl_info), l_txt, 1, 2, 0, 1, GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);

	ret=ctime(&info.st_ctime);
	memccpy(str_ctime, ret, '\0', 29);
	ret=ctime(&info.st_atime);
	memccpy(str_atime, ret, '\0', 29);
	ret=ctime(&info.st_mtime);
	memccpy(str_mtime, ret, '\0', 29);
	sprintf(str, "\ndate de changement:   %s"
		     "date d'acces:         %s"
		     "date de modification: %s", str_ctime, str_atime, str_mtime);
		
	l_info=gtk_label_new(str);
	gtk_label_set_justify (GTK_LABEL(l_info), GTK_JUSTIFY_LEFT);
	gtk_box_pack_start(GTK_BOX(vbox), l_info, TRUE, FALSE, 5);
	gtk_widget_show(l_info);
	gtk_widget_map(l_info);

	/* Affiche les permissions */
	tbl=gtk_table_new(4, 4, TRUE);
	gtk_box_pack_start(GTK_BOX(vbox), tbl, TRUE, FALSE, 0);
	gtk_widget_show(tbl);

	permission_table(tbl, 0);

	button_box(vbox, str_nom, dir_change_info);

	gtk_widget_show(propwin);

	if(S_ISDIR(info.st_mode)) {
		curs=gdk_cursor_new(GDK_WATCH);
		gdk_window_set_cursor (propwin->window, curs);
		total=0;
		gtk_widget_show(l_total);
		gtk_widget_show(l_txt);
		size_subdir(str_nom);
	}
	gtk_signal_connect (GTK_OBJECT (propwin), "delete_event",
                               GTK_SIGNAL_FUNC (close_info), NULL);
	gtk_widget_set_sensitive(propwin, TRUE);
	curs=gdk_cursor_new(GDK_LEFT_PTR);
	gdk_window_set_cursor (propwin->window, curs);
}

/*******************************************************************/
/*******************************************************************/

static void size_subdir(char *path) {
	DIR *dir;
	struct dirent *file;
	struct stat f_info;
	int ret;
	gchar *current, str[50];

	if((current=my_getcwd())==NULL) {
		perror("getcwd");
		exit(1);
	}
	
	if((dir=opendir(path))==NULL) {
		return;
	}
	
	chdir(path);
		
	while((file=readdir(dir))!=NULL) {
		if((ret=lstat(file->d_name, &f_info))==-1)
		      continue;		/* ici on ignore les erreurs */
		else {
			total+=f_info.st_size;
			if( strcmp(".", file->d_name)!=0 && strcmp("..", file->d_name)!=0 && S_ISDIR(f_info.st_mode) ) {
				nb_dir++;
				size_subdir(file->d_name);
			} else nb_file++;
			 
		}
		sprintf(str, "%ld", total);
		gtk_label_set_text(GTK_LABEL(l_total), str);
	}
	while (gtk_events_pending()) gtk_main_iteration();
	
	if(closedir(dir)==-1) perror("closedir");
	chdir(current);
	if(current!=NULL) free(current);
}

/*******************************************************************/
/*******************************************************************/

static int recursive_chmod(char *path) {
	DIR *dir;
	struct dirent *file;
	struct stat f_info;
	int ret, lnk=0;
	gchar *current;

	if((current=my_getcwd())==NULL) {
		perror("getcwd");
		exit(1);
	}
	
	if((dir=opendir(path))==NULL) {
		multierror_dialog(path, strerror(errno));
		return lnk;
	}
	
	chdir(path);
		
	while((file=readdir(dir))!=NULL) {
		if((ret=lstat(file->d_name, &f_info))==-1) {
			multierror_dialog(file->d_name, strerror(errno));
			continue;		/* ici on ignore les erreurs */
		}
		else {
/*			printf("%s %d\n", file->d_name, S_ISLNK(info.st_mode));*/
			if(S_ISLNK(f_info.st_mode))
				lnk++;
			else {
				switch(op) {
				    case 0:
					if(chmod(file->d_name, info.st_mode|f_info.st_mode)==-1)
						multierror_dialog(file->d_name, strerror(errno));
					break;
				    case 1:
					if(chmod(file->d_name, f_info.st_mode&(~info.st_mode))==-1)
						multierror_dialog(file->d_name, strerror(errno));
					break;
				    case 2:
					if(chmod(file->d_name, info.st_mode)==-1)
						multierror_dialog(file->d_name, strerror(errno));
					break;
				}
				if( strcmp(".", file->d_name)!=0
				    && strcmp("..", file->d_name)!=0
				    && S_ISDIR(f_info.st_mode) ) {
					lnk+=recursive_chmod(file->d_name);
				}			 
			}
		}
	}
	
	if(closedir(dir)==-1) perror("closedir");
	chdir(current);
	if(current!=NULL) free(current);

	return lnk;
}
